@@ -1,27 +1,24 @@
 /*
- * local defines for the low level memory routines
+ * Local defines for the low level memory routines
  *
- * Copyright 1995 by Gray Watson
+ * Copyright 1999 by Gray Watson
  *
  * This file is part of the dmalloc package.
  *
- * Permission to use, copy, modify, and distribute this software for any
- * NON-COMMERCIAL purpose and without fee is hereby granted, provided
- * that the above copyright notice and this permission notice appear
- * in all copies, and that the name of Gray Watson not be used in
- * advertising or publicity pertaining to distribution of the document
- * or software without specific, written prior permission.
- *
- * Please see the PERMISSIONS file or contact the author for information
- * about commercial licenses.
+ * Permission to use, copy, modify, and distribute this software for
+ * any purpose and without fee is hereby granted, provided that the
+ * above copyright notice and this permission notice appear in all
+ * copies, and that the name of Gray Watson not be used in advertising
+ * or publicity pertaining to distribution of the document or software
+ * without specific, written prior permission.
  *
  * Gray Watson makes no representations about the suitability of the
  * software described herein for any purpose.  It is provided "as is"
  * without express or implied warranty.
  *
- * The author may be contacted at gray.watson@letters.com
+ * The author may be contacted via http://www.dmalloc.com/
  *
- * $Id: chunk_loc.h,v 1.46 1997/12/08 09:50:52 gray Exp $
+ * $Id: chunk_loc.h,v 1.58 1999/03/10 21:35:43 gray Exp $
  */
 
 #ifndef __CHUNK_LOC_H__
@@ -31,28 +28,24 @@
 #include "dmalloc_loc.h"			/* for DMALLOC_SIZE */
 
 /* for thread-id types -- see conf.h */
-#if STORE_THREAD_ID
+#if LOG_THREAD_ID
 #ifdef THREAD_INCLUDE
 #include THREAD_INCLUDE
 #endif
 #endif
 
-/* for timeval type -- see conf.h */
+/* for time type -- see settings.h */
 #if STORE_TIMEVAL
-#ifdef TIMEVAL_INCLUDE
-#include TIMEVAL_INCLUDE
+# ifdef TIMEVAL_INCLUDE
+#  include TIMEVAL_INCLUDE
+# endif
+#else
+# if STORE_TIME
+#  ifdef TIME_INCLUDE
+#   include TIME_INCLUDE
+#  endif
+# endif
 #endif
-#endif
-
-/* defines for the malloc subsystem */
-
-/* checking information */
-#define MIN_FILE_LENGTH		    3		/* min "[a-zA-Z].c" length */
-#define MAX_FILE_LENGTH		   40		/* max __FILE__ length */
-#define MAX_LINE_NUMBER		10000		/* max __LINE__ value */
-
-/* free info */
-#define BLANK_CHAR		'\305'		/* to blank free space with */
 
 /* log-bad-space info */
 #define SPECIAL_CHARS		"\"\"''\\\\n\nr\rt\tb\bf\fa\007"
@@ -76,8 +69,8 @@
 #define BLOCK_NUM_TO_PNT(pnt)	(((long)(pnt) / BLOCK_SIZE) * BLOCK_SIZE)
 
 /* adjust internal PNT to user-space */
-#define CHUNK_TO_USER(pnt)	((char *)(pnt) + pnt_fence_bottom)
-#define USER_TO_CHUNK(pnt)	((char *)(pnt) - pnt_fence_bottom)
+#define CHUNK_TO_USER(pnt)	((char *)(pnt) + fence_bottom_size)
+#define USER_TO_CHUNK(pnt)	((char *)(pnt) - fence_bottom_size)
 
 /* get the number of blocks to hold SIZE */
 #define NUM_BLOCKS(size)	(((size) + (BLOCK_SIZE - 1)) / BLOCK_SIZE)
@@ -114,8 +107,8 @@
 				  (ans) = _b; \
 				} while (0)
   
-/* NOTE: FENCE_TOP defined in conf.h */
-#define FENCE_OVERHEAD		(FENCE_BOTTOM + FENCE_TOP)
+/* NOTE: FENCE_BOTTOM_SIZE and TOP_SIZE defined in settings.h */
+#define FENCE_OVERHEAD_SIZE	(FENCE_BOTTOM_SIZE + FENCE_TOP_SIZE)
 #define FENCE_MAGIC_BOTTOM	0xC0C0AB1B
 #define FENCE_MAGIC_TOP		0xFACADE69
 /* type of the fence magic values */
@@ -123,9 +116,10 @@
 
 #define FENCE_WRITE(pnt, size)	do { \
 				  memcpy((char *)(pnt), fence_bottom, \
-					FENCE_BOTTOM); \
-				  memcpy((char *)(pnt) + (size) - FENCE_TOP, \
-					 fence_top, FENCE_TOP); \
+					FENCE_BOTTOM_SIZE); \
+				  memcpy((char *)(pnt) + (size) \
+					 - FENCE_TOP_SIZE, \
+					 fence_top, FENCE_TOP_SIZE); \
 				} while (0)
 
 #define CHUNK_MAGIC_BOTTOM	0xDEA007	/* bottom magic number */
@@ -140,8 +134,11 @@
 #define BBLOCK_DBLOCK_ADMIN	0x0010		/* pointing to dblock admin */
 #define BBLOCK_FREE		0x0020		/* block is free */
 #define BBLOCK_EXTERNAL		0x0040		/* externally used block */
-#define BBLOCK_ADMIN_FREE	0x0080		/* ba_freen pnt to free slot */
+#define BBLOCK_ADMIN_FREE	0x0080		/* ba_free_n pnt to free slot*/
 #define BBLOCK_STRING		0x0100		/* block is a string (unused)*/
+#define BBLOCK_VALLOC		0x0200		/* block is aligned alloc */
+
+#define BBLOCK_FLAG_TYPE(f)	((f) & BBLOCK_ALLOCATED)
 
 /*
  * some overhead fields for recording information about the pointer.
@@ -151,22 +148,23 @@
  */
 typedef struct {
 #if STORE_SEEN_COUNT
-  int		ov_seenc;		/* times pointer was seen */
+  unsigned long	ov_seen_c;		/* times pointer was seen */
 #endif
 #if STORE_ITERATION_COUNT
-  long		ov_iteration;		/* interation when pointer alloced */
-#endif
-#if STORE_TIME
-  long		ov_time;		/* time when pointer alloced */
+  unsigned long	ov_iteration;		/* interation when pointer alloced */
 #endif
 #if STORE_TIMEVAL
-  struct timeval  ov_timeval;		/* time when pointer alloced */
+  TIMEVAL_TYPE  ov_timeval;		/* time when pointer alloced */
+#else
+#if STORE_TIME
+  TIME_TYPE	ov_time;		/* time when pointer alloced */
 #endif
-#if STORE_THREAD_ID
+#endif
+#if LOG_THREAD_ID
   THREAD_TYPE	ov_thread_id;		/* thread id which allocaed pnt */
 #endif
   
-#if STORE_SEEN_COUNT == 0 && STORE_ITERATION_COUNT == 0 && STORE_TIME == 0 && STORE_TIMEVAL == 0 && STORE_THREAD_ID == 0
+#if STORE_SEEN_COUNT == 0 && STORE_ITERATION_COUNT == 0 && STORE_TIME == 0 && STORE_TIMEVAL == 0 && LOG_THREAD_ID == 0
   int		ov_junk;		/* for compilers that hate 0 structs */
 #endif
 } overhead_t;
@@ -188,7 +186,7 @@ typedef struct {
  * when the pointer is Alloc[at]ed.  The db_next field is used when
  * the admin structure is on the free list.
  */
-struct dblock_st {
+typedef struct dblock_st {
   union {
     struct {
       unsigned short	nu_size;		/* size of contiguous area */
@@ -212,9 +210,12 @@ struct dblock_st {
 #define db_next		db_pnt.pn_next		/* Free */
 #define db_file		db_pnt.pn_file		/* Alloced */
   
+#if FREED_POINTER_DELAY
+  unsigned long	db_reuse_iter;			/* when avail for reuse */
+#endif
+  
   overhead_t	db_overhead;			/* configured overhead adds */
-};
-typedef struct dblock_st	dblock_t;
+} dblock_t;
 
 /*
  * Below defines a basic-block structure.  This structure is used to
@@ -226,11 +227,11 @@ typedef struct dblock_st	dblock_t;
  * simplify the code and the comments at the end show when the field
  * is used.  For instance, bb_bit_n (really bb_nums.nu_bit_n) is in use
  * when the pointer is tracking user divided-block allocations or when
- * the pointer is free.  The bb_freen field is used when the block is
+ * the pointer is free.  The bb_free_n field is used when the block is
  * full of admin structures such as this.  Yes, the library uses this
  * structure to track another block full of these structures.  Yikes!
  */
-struct bblock_st {
+typedef struct bblock_st {
   unsigned short	bb_flags;		/* what it is */
   
   union {
@@ -239,12 +240,12 @@ struct bblock_st {
   } bb_nums;
   
   /* to reference union elements as bb elements */
-#define bb_bit_n		bb_nums.nu_bit_n		/* User-dblock, Free */
+#define bb_bit_n	bb_nums.nu_bit_n	/* User-dblock, Free */
 #define bb_line		bb_nums.nu_line		/* User-bblock */
   
   union {
-    unsigned long	in_freen;		/* admin free number */
-    unsigned long	in_posn;		/* admin block position */
+    unsigned long	in_free_n;		/* admin free number */
+    unsigned long	in_pos_n;		/* admin block position */
     unsigned long	in_block_n;		/* number of blocks */
     unsigned long	in_size;		/* size of allocation */
     /* NOTE: this pointer and the longs may be of a different type */
@@ -252,14 +253,14 @@ struct bblock_st {
   } bb_info;
   
   /* to reference union elements as bb elements */
-#define bb_freen	bb_info.in_freen	/* BBlock-admin-free */
-#define	bb_posn		bb_info.in_posn		/* BBlock-admin */
+#define bb_free_n	bb_info.in_free_n	/* BBlock-admin-free */
+#define	bb_pos_n	bb_info.in_pos_n	/* BBlock-admin */
 #define	bb_block_n	bb_info.in_block_n	/* Free */
 #define	bb_size		bb_info.in_size		/* User-bblock */
 #define	bb_dblock	bb_info.in_dblock	/* User-dblock */
   
   union {
-    struct dblock_adm_st	*pn_slotp;	/* pointer to db_admin block */
+    struct dblock_adm_st	*pn_slot_p;	/* pointer to db_admin block */
     struct bblock_adm_st	*pn_admin_p;	/* pointer to bb_admin block */
     void			*pn_mem;	/* memory associated to it */
     struct bblock_st		*pn_next;	/* next in free list */
@@ -267,40 +268,41 @@ struct bblock_st {
   } bb_pnt;
   
   /* to reference union elements as bb elements */
-#define	bb_slotp	bb_pnt.pn_slotp		/* DBlock-admin */
+#define	bb_slot_p	bb_pnt.pn_slot_p	/* DBlock-admin */
 #define	bb_admin_p	bb_pnt.pn_admin_p	/* BBlock-admin */
 #define	bb_mem		bb_pnt.pn_mem		/* User-dblock, External */
 #define	bb_next		bb_pnt.pn_next		/* Free */
 #define	bb_file		bb_pnt.pn_file		/* User-bblock */
   
+#if FREED_POINTER_DELAY
+  unsigned long	bb_reuse_iter;			/* when avail for reuse */
+#endif
+  
   overhead_t	bb_overhead;			/* configured overhead adds */
-};
-typedef struct bblock_st	bblock_t;
+} bblock_t;
 
 /*
  * Below is a definition of a bblock (basic-block) administration
  * structure.  This structure which fills a basic-block of space in
  * memory and holds a number of bblock structures.
  */
-struct bblock_adm_st {
+typedef struct bblock_adm_st {
   long			ba_magic1;		/* bottom magic number */
-  unsigned int		ba_posn;		/* position in bblock array */
+  unsigned int		ba_pos_n;		/* position in bblock array */
   bblock_t		ba_blocks[BB_PER_ADMIN]; /* bblock admin info */
   struct bblock_adm_st	*ba_next;		/* next bblock adm struct */
   long			ba_magic2;		/* top magic number */
-};
-typedef struct bblock_adm_st	bblock_adm_t;
+} bblock_adm_t;
 
 /*
  * Below is a definition of a dblock (divided-block) administration
  * structure.  This structure fills a basic-block of space in memory
  * and holds a number of dblock structures.
  */
-struct dblock_adm_st {
+typedef struct dblock_adm_st {
   long			da_magic1;		/* bottom magic number */
   dblock_t		da_block[DB_PER_ADMIN];	/* dblock admin info */
   long			da_magic2;		/* top magic number */
-};
-typedef struct dblock_adm_st	dblock_adm_t;
+} dblock_adm_t;
 
 #endif /* ! __CHUNK_LOC_H__ */
