@@ -1,27 +1,24 @@
 /*
- * token attributes for the dmalloc program and _dmalloc_flags
+ * Token attributes for the dmalloc program and _dmalloc_flags
  *
- * Copyright 1995 by Gray Watson
+ * Copyright 1999 by Gray Watson
  *
  * This file is part of the dmalloc package.
  *
- * Permission to use, copy, modify, and distribute this software for any
- * NON-COMMERCIAL purpose and without fee is hereby granted, provided
- * that the above copyright notice and this permission notice appear
- * in all copies, and that the name of Gray Watson not be used in
- * advertising or publicity pertaining to distribution of the document
- * or software without specific, written prior permission.
- *
- * Please see the PERMISSIONS file or contact the author for information
- * about commercial licenses.
+ * Permission to use, copy, modify, and distribute this software for
+ * any purpose and without fee is hereby granted, provided that the
+ * above copyright notice and this permission notice appear in all
+ * copies, and that the name of Gray Watson not be used in advertising
+ * or publicity pertaining to distribution of the document or software
+ * without specific, written prior permission.
  *
  * Gray Watson makes no representations about the suitability of the
  * software described herein for any purpose.  It is provided "as is"
  * without express or implied warranty.
  *
- * The author may be contacted at gray.watson@letters.com
+ * The author may be contacted via http://www.dmalloc.com/
  *
- * $Id: debug_tok.h,v 1.23 1997/12/07 23:17:30 gray Exp $
+ * $Id: debug_tok.h,v 1.29 1999/03/04 19:08:24 gray Exp $
  */
 
 #ifndef __DEBUG_TOK_H__
@@ -49,12 +46,12 @@ static	attr_t		attributes[] = {
       "log general statistics" },
   { "log-non-free",	"lnf",	DEBUG_LOG_NONFREE,
       "log non-freed pointers" },
-  { "log-thread-id",	"lti",	DEBUG_LOG_THREAD_ID,
-      "log thread-id for allocated pointer" },
+  { "log-thread-id",	"lti",	0,
+      "Disabled token -- see LOG_THREAD_ID in settings.h" },
   { "log-trans",	"ltr",	DEBUG_LOG_TRANS,
       "log memory transactions" },
-  { "log-stamp",	"lst",	DEBUG_LOG_STAMP,
-      "add time-stamp to log" },
+  { "log-stamp",	"lst",	0,
+      "Disabled token -- see LOG_TIME_NUMBER in settings.h" },
   { "log-admin",	"lad",	DEBUG_LOG_ADMIN,
       "log administrative info" },
   { "log-blocks",	"lbl",	DEBUG_LOG_BLOCKS,
@@ -78,35 +75,40 @@ static	attr_t		attributes[] = {
   { "check-lists",	"cli",	DEBUG_CHECK_LISTS,
       "check free lists" },
   { "check-blank",	"cbl",	DEBUG_CHECK_BLANK,
-      "check blanked memory" },
+      "check mem overwritten by alloc-blank, free-blank" },
   { "check-funcs",	"cfu",	DEBUG_CHECK_FUNCS,
       "check functions" },
   
   { "catch-signals",	"csi",	DEBUG_CATCH_SIGNALS,
-      "shutdown lib on SIGHUP, SIGINT, SIGTERM"},
-  
+      "shutdown program on SIGHUP, SIGINT, SIGTERM" },
   { "realloc-copy",	"rco",	DEBUG_REALLOC_COPY,
       "copy all re-allocations" },
   { "free-blank",	"fbl",	DEBUG_FREE_BLANK,
-      "blank free'd memory" },
+      "overwrite freed memory space with BLANK_CHAR" },
   { "error-abort",	"eab",	DEBUG_ERROR_ABORT,
       "abort immediately on error" },
   { "alloc-blank",	"abl",	DEBUG_ALLOC_BLANK,
-      "blank newly alloced memory" },
+      "overwrite newly alloced memory with BLANK_CHAR" },
   { "heap-check-map",	"hcm",	DEBUG_HEAP_CHECK_MAP,
       "log heap-map on heap-check" },
-  { "print-error",	"per",	DEBUG_PRINT_ERROR,
-      "print errors to stderr" },
+  { "print-messages",	"pme",	DEBUG_PRINT_MESSAGES,
+      "write messages to stderr" },
+  /* OLD VERSION -- can be removed later */
+  { "print-error",	"per",	DEBUG_PRINT_MESSAGES,
+      "old version -- please use print-messages" },
   { "catch-null",	"cnu",	DEBUG_CATCH_NULL,
       "abort if no memory available" },
   { "never-reuse",	"nre",	DEBUG_NEVER_REUSE,
       "never re-use freed memory" },
   { "allow-nonlinear",	"ano",	DEBUG_ALLOW_NONLINEAR,
-      "allow non-linear heap space"},
-  { "allow-zero",	"aze",	DEBUG_ALLOW_ZERO,
-      "allow allocs of 0 bytes, frees of NULL"},
+      "allow non-linear heap space" },
+  { "allow-free-null",	"afn",	DEBUG_ALLOW_FREE_NULL,
+      "allow the frees of NULL pointers" },
+  /* OLD VERSION -- can be removed later */
+  { "allow-zero",	"aze",	DEBUG_ALLOW_FREE_NULL,
+      "old version -- please use allow-free-null" },
   { "error-dump",	"edu",	DEBUG_ERROR_DUMP,
-      "dump core on error and then continue"},
+      "dump core on error and then continue" },
   
   { NULL }
 };
