@@ -1,27 +1,24 @@
 /*
- * debug values for DMALLOC_DEBUG and _dmalloc_flags
+ * Debug values for DMALLOC_DEBUG and _dmalloc_flags
  *
- * Copyright 1995 by Gray Watson
+ * Copyright 1999 by Gray Watson
  *
  * This file is part of the dmalloc package.
  *
- * Permission to use, copy, modify, and distribute this software for any
- * NON-COMMERCIAL purpose and without fee is hereby granted, provided
- * that the above copyright notice and this permission notice appear
- * in all copies, and that the name of Gray Watson not be used in
- * advertising or publicity pertaining to distribution of the document
- * or software without specific, written prior permission.
- *
- * Please see the PERMISSIONS file or contact the author for information
- * about commercial licenses.
+ * Permission to use, copy, modify, and distribute this software for
+ * any purpose and without fee is hereby granted, provided that the
+ * above copyright notice and this permission notice appear in all
+ * copies, and that the name of Gray Watson not be used in advertising
+ * or publicity pertaining to distribution of the document or software
+ * without specific, written prior permission.
  *
  * Gray Watson makes no representations about the suitability of the
  * software described herein for any purpose.  It is provided "as is"
  * without express or implied warranty.
  *
- * The author may be contacted at gray.watson@letters.com
+ * The author may be contacted via http://www.dmalloc.com/
  *
- * $Id: debug_val.h,v 1.17 1997/03/21 21:22:38 gray Exp $
+ * $Id: debug_val.h,v 1.22 1999/03/04 19:08:25 gray Exp $
  */
 
 #ifndef __DEBUG_VAL_H__
@@ -38,9 +35,9 @@
 /* logging */
 #define DEBUG_LOG_STATS		BIT_FLAG(0)	/* generally log statistics */
 #define DEBUG_LOG_NONFREE	BIT_FLAG(1)	/* report non-freed pointers */
-#define DEBUG_LOG_THREAD_ID	BIT_FLAG(2)	/* log pnt thread-id info */
+/* 2 available */
 #define DEBUG_LOG_TRANS		BIT_FLAG(3)	/* log memory transactions */
-#define DEBUG_LOG_STAMP		BIT_FLAG(4)	/* add a time stamp to log */
+/* 4 available */
 #define DEBUG_LOG_ADMIN		BIT_FLAG(5)	/* log background admin info */
 #define DEBUG_LOG_BLOCKS	BIT_FLAG(6)	/* log blocks when heap-map */
 #define DEBUG_LOG_UNKNOWN	BIT_FLAG(7)	/* report unknown non-freed */
@@ -56,20 +53,22 @@
 #define DEBUG_CHECK_LISTS	BIT_FLAG(12)	/* check the free lists */
 #define DEBUG_CHECK_BLANK	BIT_FLAG(13)	/* check blank sections */
 #define DEBUG_CHECK_FUNCS	BIT_FLAG(14)	/* check functions */
+/* 15 available */
+/* 16 available */
 
 /* misc */
 #define DEBUG_CATCH_SIGNALS	BIT_FLAG(17)	/* catch HUP, INT, and TERM */
-/* 18,19 taken above */
+/* 18,19 used above */
 #define DEBUG_REALLOC_COPY	BIT_FLAG(20)	/* copy all reallocations */
 #define DEBUG_FREE_BLANK	BIT_FLAG(21)	/* write over free'd memory */
 #define DEBUG_ERROR_ABORT	BIT_FLAG(22)	/* abort on error else exit */
 #define DEBUG_ALLOC_BLANK	BIT_FLAG(23)	/* write over to-be-alloced */
 #define DEBUG_HEAP_CHECK_MAP	BIT_FLAG(24)	/* heap-map on heap-check */
-#define DEBUG_PRINT_ERROR	BIT_FLAG(25)	/* print errors to STDERR */
+#define DEBUG_PRINT_MESSAGES	BIT_FLAG(25)	/* write messages to STDERR */
 #define DEBUG_CATCH_NULL	BIT_FLAG(26)	/* quit before return null */
 #define DEBUG_NEVER_REUSE	BIT_FLAG(27)	/* never reuse memory */
 #define DEBUG_ALLOW_NONLINEAR	BIT_FLAG(28)	/* allow non-linear heap */
-#define DEBUG_ALLOW_ZERO	BIT_FLAG(29)	/* allow alloc(0) and free(0)*/
+#define DEBUG_ALLOW_FREE_NULL	BIT_FLAG(29)	/* allow free(0)*/
 #define DEBUG_ERROR_DUMP	BIT_FLAG(30)	/* dump core on error */
 /* 31 is the high bit and off-limits */
 
