@@ -1,25 +1,24 @@
 /*
  * program that handles the dmalloc variables.
  *
- * Copyright 1995 by Gray Watson
+ * Copyright 1999 by Gray Watson
  *
  * This file is part of the dmalloc package.
  *
- * Permission to use, copy, modify, and distribute this software for any
- * NON-COMMERCIAL purpose and without fee is hereby granted, provided
- * that the above copyright notice and this permission notice appear
- * in all copies, and that the name of Gray Watson not be used in
- * advertising or publicity pertaining to distribution of the document
- * or software without specific, written prior permission.
- *
- * Please see the PERMISSIONS file or contact the author for information
- * about commercial licenses.
+ * Permission to use, copy, modify, and distribute this software for
+ * any purpose and without fee is hereby granted, provided that the
+ * above copyright notice and this permission notice appear in all
+ * copies, and that the name of Gray Watson not be used in advertising
+ * or publicity pertaining to distribution of the document or software
+ * without specific, written prior permission.
  *
  * Gray Watson makes no representations about the suitability of the
  * software described herein for any purpose.  It is provided "as is"
  * without express or implied warranty.
  *
- * The author may be contacted at gray.watson@letters.com
+ * The author may be contacted via http://www.dmalloc.com/
+ *
+ * $Id: dmalloc.c,v 1.88 1999/05/06 15:11:46 gray Exp $
  */
 
 /*
@@ -59,16 +58,22 @@
 #include "version.h"
 
 #if INCLUDE_RCS_IDS
+#ifdef __GNUC__
+#ident "$Id: dmalloc.c,v 1.88 1999/05/06 15:11:46 gray Exp $";
+#else
 static	char	*rcs_id =
-  "$Id: dmalloc.c,v 1.74 1997/12/22 00:24:52 gray Exp $";
+  "$Id: dmalloc.c,v 1.88 1999/05/06 15:11:46 gray Exp $";
+#endif
 #endif
 
 #define HOME_ENVIRON	"HOME"			/* home directory */
 #define SHELL_ENVIRON	"SHELL"			/* for the type of shell */
-#define DEFAULT_CONFIG	"%s/.dmallocrc"		/* default config file */
-#define TOKENIZE_CHARS	" \t,="			/* for tag lines */
+#define DEFAULT_CONFIG	".dmallocrc"		/* default config file */
+#define TOKENIZE_EQUALS	" \t="			/* for tag lines */
+#define TOKENIZE_CHARS	" \t,"			/* for tag lines */
 
 #define NO_VALUE		(-1)		/* no value ... value */
+#define INTERVAL_NO_VALUE	0		/* no value ... value */
 #define TOKENS_PER_LINE		5		/* num debug toks per line */
 
 /*
@@ -93,8 +98,8 @@ typedef struct {
 #define HIGH_FLAGS	(MEDIUM_FLAGS | \
 			 DEBUG_CHECK_BLANK | DEBUG_CHECK_FUNCS)
 #define ALL_FLAGS	(HIGH_FLAGS | \
-			 DEBUG_LOG_TRANS | DEBUG_LOG_STAMP | \
-			 DEBUG_LOG_ADMIN | DEBUG_LOG_BLOCKS | \
+			 DEBUG_LOG_TRANS | DEBUG_LOG_ADMIN | \
+			 DEBUG_LOG_BLOCKS | \
 			 DEBUG_HEAP_CHECK_MAP | DEBUG_NEVER_REUSE)
 /* NOTE: print-error is not in this list because it is special */
 
@@ -116,8 +121,9 @@ static	char	cshell_b	= FALSE;	/* set c-shell output */
 static	char	clear_b		= FALSE;	/* clear variables */
 static	int	debug		= NO_VALUE;	/* for DEBUG */
 static	int	errno_to_print	= NO_VALUE;	/* to print the error string */
+static	char	gdb_b		= FALSE;	/* set gdb output */
 static	char	*inpath		= NULL;		/* for config-file path */
-static	int	interval	= NO_VALUE;	/* for setting INTERVAL */
+static	unsigned long interval	= INTERVAL_NO_VALUE; /* for setting INTERVAL */
 static	int	thread_lock_on	= NO_VALUE;	/* for setting LOCK_ON */
 static	char	keep_b		= FALSE;	/* keep settings override -r */
 static	char	list_tags_b	= FALSE;	/* list rc tags */
@@ -125,7 +131,7 @@ static	char	debug_tokens_b	= FALSE;	/* list debug tokens */
 static	char	*logpath	= NULL;		/* for LOGFILE setting */
 static	char	long_tokens_b	= FALSE;	/* long-tok output */
 static	argv_array_t	minus;			/* tokens to remove */
-static	char	no_changes_b	= FALSE;	/* make no changes to env */
+static	char	make_changes_b	= TRUE;		/* make no changes to env */
 static	argv_array_t	plus;			/* tokens to add */
 static	char	remove_auto_b	= FALSE;	/* auto-remove settings */
 static	char	short_tokens_b	= FALSE;	/* short-tok output */
@@ -140,11 +146,16 @@ static	argv_t	args[] = {
   { ARGV_OR },
   { 'C',	"c-shell",	ARGV_BOOL,	&cshell_b,
       NULL,			"set output for C-type shells" },
+  { ARGV_OR },
+  { 'g',	"gdb",		ARGV_BOOL,	&gdb_b,
+      NULL,			"set output for gdb parsing" },
+  
   { 'L',	"long-tokens",	ARGV_BOOL,	&long_tokens_b,
       NULL,			"output long-tokens not 0x..." },
   { ARGV_OR },
   { 'S',	"short-tokens",	ARGV_BOOL,	&short_tokens_b,
       NULL,			"output short-tokens not 0x..." },
+  
   { 'a',	"address",	ARGV_CHAR_P,	&address,
       "address:#",		"stop when malloc sees address" },
   { 'c',	"clear",	ARGV_BOOL,	&clear_b,
@@ -157,7 +168,7 @@ static	argv_t	args[] = {
       "errno",			"print error string for errno" },
   { 'f',	"file",		ARGV_CHAR_P,	&inpath,
       "path",			"config if not ~/.mallocrc" },
-  { 'i',	"interval",	ARGV_INT,	&interval,
+  { 'i',	"interval",	ARGV_U_LONG,	&interval,
       "value",			"check heap every number times" },
   { 'k',	"keep",		ARGV_BOOL,	&keep_b,
       NULL,			"keep settings (override -r)" },
@@ -165,7 +176,7 @@ static	argv_t	args[] = {
       "path",			"file to log messages to" },
   { 'm',	"minus",	ARGV_CHAR_P | ARGV_FLAG_ARRAY,	&minus,
       "token(s)",		"del tokens from current debug" },
-  { 'n',	"no-changes",	ARGV_BOOL,	&no_changes_b,
+  { 'n',	"no-changes",	ARGV_BOOL_NEG,	&make_changes_b,
       NULL,			"make no changes to the env" },
   { 'o',	"lock-on",	ARGV_INT,	&thread_lock_on,
       "number",			"number of times to not lock" },
@@ -197,25 +208,26 @@ static	char	*sh_shells[] = { "sh", "ash", "bash", "ksh", "zsh", NULL };
  */
 static	void	choose_shell(void)
 {
-  const char	*shell, *shellp;
+  const char	*shell, *shell_p;
   int		shell_c;
   
   shell = getenv(SHELL_ENVIRON);
   if (shell == NULL) {
+    /* oh well, we just guess on c-shell */
     cshell_b = TRUE;
     return;
   }
   
-  shellp = strrchr(shell, '/');
-  if (shellp == NULL) {
-    shellp = shell;
+  shell_p = strrchr(shell, '/');
+  if (shell_p == NULL) {
+    shell_p = shell;
   }
   else {
-    shellp++;
+    shell_p++;
   }
   
   for (shell_c = 0; sh_shells[shell_c] != NULL; shell_c++) {
-    if (strcmp(sh_shells[shell_c], shellp) == 0) {
+    if (strcmp(sh_shells[shell_c], shell_p) == 0) {
       bourne_b = TRUE;
       return;
     }
@@ -229,25 +241,25 @@ static	void	choose_shell(void)
  */
 static	void	dump_debug(const int val)
 {
-  attr_t	*attrp;
+  attr_t	*attr_p;
   int		tok_c = 0, work = val;
   
-  for (attrp = attributes; attrp->at_string != NULL; attrp++) {
+  for (attr_p = attributes; attr_p->at_string != NULL; attr_p++) {
     /* the below is necessary to handle the 'none' HACK */
-    if ((work == 0 && attrp->at_value == 0)
-	|| (attrp->at_value != 0
-	    && (work & attrp->at_value) == attrp->at_value)) {
-      work &= ~attrp->at_value;
+    if ((work == 0 && attr_p->at_value == 0)
+	|| (attr_p->at_value != 0
+	    && BIT_IS_SET(work, attr_p->at_value))) {
+      BIT_CLEAR(work, attr_p->at_value);
       
       if (tok_c == 0) {
 	(void)fprintf(stderr, "   ");
       }
       
       if (very_verbose_b) {
-	(void)fprintf(stderr, "%s -- %s", attrp->at_string, attrp->at_desc);
+	(void)fprintf(stderr, "%s -- %s", attr_p->at_string, attr_p->at_desc);
       }
       else {
-	(void)fprintf(stderr, "%s", attrp->at_string);
+	(void)fprintf(stderr, "%s", attr_p->at_string);
 	if (work != 0) {
 	  (void)fprintf(stderr, ", ");
 	}
@@ -279,67 +291,44 @@ static	void	dump_debug(const int val)
  */
 static	long	token_to_value(const char *tok)
 {
-  int	attr_c;
-  long	ret = 0;
+  attr_t	*attr_p;
   
-  for (attr_c = 0; attributes[attr_c].at_string != NULL; attr_c++) {
-    if (strcmp(tok, attributes[attr_c].at_string) == 0) {
+  /* find the matching attribute string */
+  for (attr_p = attributes; attr_p->at_string != NULL; attr_p++) {
+    if (strcmp(tok, attr_p->at_string) == 0
+	|| strcmp(tok, attr_p->at_short) == 0) {
       break;
     }
   }
   
-  if (attributes[attr_c].at_string == NULL) {
-    (void)fprintf(stderr, "%s: unknown token '%s'\n",
-		  argv_program, tok);
-  }
-  else {
-    ret = attributes[attr_c].at_value;
+  if (attr_p->at_string == NULL) {
+    (void)fprintf(stderr, "%s: unknown token '%s'\n", argv_program, tok);
+    return 0;
   }
   
-  return ret;
+  /* if we have a 0 value and not none then this is a disabled token */
+  if (attr_p->at_value == 0 && strcmp(tok, "none") != 0) {
+    (void)fprintf(stderr, "%s: token '%s' has been disabled: %s\n",
+		  argv_program, tok, attr_p->at_desc);
+    return 0;
+  }
+  
+  return attr_p->at_value;
 }
 
 /*
- * process the user configuration looking for the TAG_FIND.  if it is
- * null then look for DEBUG_VALUE in the file and return the token for
- * it in STRP.  routine returns the new debug value matching tag.
+ * Read in the next token from INFILE.  It passes back the returned
+ * debug value in DEBUG_P.  Passes back the matching TOKEN of
+ * TOKEN_SIZE.  Returns 1 if there was a next else 0.
  */
-static	long	process(const long debug_value, const char *tag_find,
-			char **strp)
+static	int	read_next_token(FILE *infile, long *debug_p,
+				char *token, const int token_size)
 {
-  static char	token[128];
-  FILE		*infile = NULL;
-  char		path[1024], buf[1024];
-  const char	*home_p;
-  int		found_b, cont_b;
-  long		new_debug = 0;
+  int	cont_b = FALSE, found_b = FALSE;
+  long	new_debug = 0;
+  char	buf[1024], *tok_p, *buf_p;
   
-  /* do we need to have a home variable? */
-  if (inpath == NULL) {
-    home_p = getenv(HOME_ENVIRON);
-    if (home_p == NULL) {
-      (void)fprintf(stderr, "%s: could not find variable '%s'\n",
-		    argv_program, HOME_ENVIRON);
-      exit(1);
-    }
-    
-    (void)sprintf(path, DEFAULT_CONFIG, home_p);
-    inpath = path;
-  }
-  
-  infile = fopen(inpath, "r");
-  /* do not test for error here */
-  
-  if (list_tags_b && infile != NULL) {
-    (void)fprintf(stderr, "Tags available from '%s':\n", inpath);
-  }
-  
-  /* read each of the lines looking for the tag */
-  found_b = FALSE;
-  cont_b = FALSE;
-  
-  while (infile != NULL && fgets(buf, sizeof(buf), infile) != NULL) {
-    char	*tok_p, *endp;
+  while (fgets(buf, sizeof(buf), infile) != NULL) {
     
     /* ignore comments and empty lines */
     if (buf[0] == '#' || buf[0] == '\n') {
@@ -347,159 +336,346 @@ static	long	process(const long debug_value, const char *tag_find,
     }
     
     /* chop off the ending \n */
-    endp = strrchr(buf, '\n');
-    if (endp != NULL) {
-      *endp = '\0';
+    buf_p = strrchr(buf, '\n');
+    if (buf_p != NULL) {
+      *buf_p = '\0';
     }
     
-    /* get the first token on the line */
-    tok_p = strtok(buf, TOKENIZE_CHARS);
-    if (tok_p == NULL) {
-      continue;
-    }
+    buf_p = buf;
     
     /* if we're not continuing then we need to process a tag */
     if (! cont_b) {
-      (void)strcpy(token, tok_p);
-      new_debug = 0;
       
-      if (tag_find != NULL && strcmp(tag_find, tok_p) == 0) {
-	found_b = TRUE;
-      }
-      
-      tok_p = strtok(NULL, TOKENIZE_CHARS);
+      /* get the first token on the line */
+      tok_p = strsep(&buf_p, TOKENIZE_EQUALS);
       if (tok_p == NULL) {
 	continue;
       }
+      if (*tok_p == '\0') {
+	(void)fprintf(stderr, "Invalid start of line: %s\n", buf_p);
+	continue;
+      }
+      
+      /* save the token */
+      if (token != NULL) {
+	(void)strncpy(token, tok_p, token_size);
+	token[token_size - 1] = '\0';
+      }
+      found_b = 1;
     }
     
     cont_b = FALSE;
     
-    do {
-      /* do we have a continuation character */
+    while (1) {
+      /* get the next token */
+      tok_p = strsep(&buf_p, TOKENIZE_CHARS);
+      if (tok_p == NULL) {
+	break;
+      }
+      if (*tok_p == '\0') {
+	continue;
+      }
+      
+      /* do we have a continuation character? */
       if (strcmp(tok_p, "\\") == 0) {
 	cont_b = TRUE;
 	break;
       }
       
-      /* are we processing the tag of choice? */
-      if (found_b || tag_find == NULL) {
-	new_debug |= token_to_value(tok_p);
-      }
-      
-      tok_p = strtok(NULL, TOKENIZE_CHARS);
-    } while (tok_p != NULL);
-    
-    if (list_tags_b && (! cont_b)) {
-      if (verbose_b) {
-	(void)fprintf(stderr, "%s (%#lx):\n", token, new_debug);
-	dump_debug(new_debug);
-      }
-      else
-	(void)fprintf(stderr, "%s\n", token);
-    }
-    else if (tag_find == NULL && (! cont_b) && new_debug == debug_value) {
-      found_b = TRUE;
-      if (strp != NULL) {
-	*strp = token;
-      }
+      new_debug |= token_to_value(tok_p);
     }
     
     /* are we done? */
-    if (found_b && (! cont_b)) {
+    if (! cont_b) {
       break;
     }
   }
   
-  if (infile != NULL) {
-    (void)fclose(infile);
+  if (debug_p != NULL) {
+    *debug_p = new_debug;
   }
   
-  /* did we find the correct value in the file? */
   if (found_b) {
-    return new_debug;
+    return 1;
+  }
+  else {
+    return 0;
+  }
+}
+
+/*
+ * Read in a rc file from PATH and process it looking for the
+ * specified DEBUG_VALUE or TAG_FIND token.  It passes back the
+ * returned debug value in DEBUG_P.  Passes back the matching TOKEN of
+ * TOKEN_SIZE.  Returns 0 if the file was not found.  Returns 1 if the
+ * file was found, 2 if the file and the token was found.
+ */
+static	int	read_rc_file(const char *path, const long debug_value,
+			     const char *tag_find, long *debug_p,
+			     char *token, const int token_size)
+{
+  FILE	*infile;
+  int	found_b = FALSE;
+  char	next_token[64];
+  long	new_debug;
+  
+  /* open the path */
+  infile = fopen(path, "r");
+  if (infile == NULL) {
+    return 0;
   }
   
-  if (! list_tags_b && tag_find == NULL) {
-    default_t	*def_p;
+  /* run through the tokens, looking for a match */
+  while (read_next_token(infile, &new_debug, next_token,
+			 sizeof(next_token)) == 1) {
     
-    /* check the default list to see if we have a match there */
-    if (strp != NULL) {
+    /* are we looking for a tag? */
+    if (tag_find != NULL && strcmp(tag_find, next_token) == 0) {
+      found_b = 1;
+      break;
+    }
+    
+    /* are we looking for a debug-value? */
+    if (debug_value > 0 && debug_value == new_debug) {
+      found_b = 1;
+      break;
+    }
+  }
+  
+  (void)fclose(infile);
+  
+  if (debug_p != NULL) {
+    *debug_p = new_debug;
+  }
+  if (token != NULL) {
+    strncpy(token, next_token, token_size);
+    token[token_size - 1] = '\0';
+  }
+  
+  if (found_b) {
+    return 2;
+  }
+  else {
+    return 1;
+  }
+}
+
+/*
+ * Process the user configuration looking for the TAG_FIND.  If it is
+ * null then look for DEBUG_VALUE in the file and copy the token found
+ * into TOKEN of TOKEN_SIZE.  Routine returns the new debug value
+ * matching tag.
+ */
+static	long	process(const long debug_value, const char *tag_find,
+			char *token, const int token_size)
+{
+  char		path[1024], *path_p;
+  default_t	*def_p;
+  const char	*home_p;
+  int		ret;
+  long		new_debug = 0;
+  
+  /* do we need to have a home variable? */
+  if (inpath == NULL) {
+    
+    /* first we try to read the RC file from the current directory */
+    ret = read_rc_file(DEFAULT_CONFIG, debug_value, tag_find, &new_debug,
+		       token, token_size);
+    /* did we find the correct value in the file? */
+    if (ret == 2) {
+      return new_debug;
+    }
+    path_p = DEFAULT_CONFIG;
+    
+    /* if we did not find the file, check the home directory */
+    if (ret == 0) {
+      /* find our home directory */
+      home_p = getenv(HOME_ENVIRON);
+      if (home_p == NULL) {
+	(void)fprintf(stderr, "%s: could not find variable '%s'\n",
+		      argv_program, HOME_ENVIRON);
+	exit(1);
+      }
+      
+      (void)loc_snprintf(path, sizeof(path), "%s/%s", home_p, DEFAULT_CONFIG);
+      
+      /* read in the file from our home directory */
+      ret = read_rc_file(path, debug_value, tag_find, &new_debug,
+			 token, token_size);
+      /* did we find the correct value in the file? */
+      if (ret == 2) {
+	return new_debug;
+      }
+      path_p = path;
+    }
+  }
+  else {
+    /* read in the specified file */
+    ret = read_rc_file(inpath, debug_value, tag_find, &new_debug,
+		       token, token_size);
+    /* did we find the correct value in the file? */
+    if (ret == 2) {
+      return new_debug;
+    }
+    /* if the specified was not found, return error */
+    if (ret == 0) {
+      (void)fprintf(stderr, "%s: could not read '%s': ",
+		    argv_program, inpath);
+      perror("");
+      exit(1);
+    }
+    path_p = inpath;
+  }
+  
+  /* if tag-find is NULL we assume we are looking for a debug-value */
+  if (tag_find == NULL) {
+    
+    /* now look for the value in the default token list */
+    if (token != NULL) {
       for (def_p = defaults; def_p->de_string != NULL; def_p++) {
 	if (def_p->de_flags == debug_value) {
-	  *strp = def_p->de_string;
+	  strncpy(token, def_p->de_string, token_size);
+	  token[token_size - 1] = '\0';
+	  new_debug = def_p->de_flags;
 	  break;
 	}
       }
       if (def_p->de_string == NULL) {
-	*strp = "unknown";
+	strncpy(token, "unknown", token_size);
+	token[token_size - 1] = '\0';
+	new_debug = 0;
       }
     }
   }
   else {
-    default_t	*defp;
     
-    if (list_tags_b) {
-      (void)fprintf(stderr, "\n");
-      (void)fprintf(stderr, "Tags available by default:\n");
-    }
-    
-    for (defp = defaults; defp->de_string != NULL; defp++) {
-      if (list_tags_b) {
-	if (verbose_b) {
-	  (void)fprintf(stderr, "%s (%#lx):\n",
-			defp->de_string, defp->de_flags);
-	  dump_debug(defp->de_flags);
-	}
-	else
-	  (void)fprintf(stderr, "%s\n", defp->de_string);
-      }
-      if (tag_find != NULL && strcmp(tag_find, defp->de_string) == 0) {
+    /* now look for the token in the default token list */
+    for (def_p = defaults; def_p->de_string != NULL; def_p++) {
+      if (strcmp(tag_find, def_p->de_string) == 0) {
+	new_debug = def_p->de_flags;
 	break;
       }
     }
-    if (! list_tags_b) {
-      if (defp->de_string == NULL) {
-	if (infile == NULL) {
-	  (void)fprintf(stderr, "%s: could not read '%s': ",
-			argv_program, inpath);
-	  perror("");
-	}
-	else {
-	  (void)fprintf(stderr, "%s: could not find tag '%s' in '%s'\n",
-			argv_program, tag_find, inpath);
-	}
-	exit(1);
-      }
-      new_debug = defp->de_flags;
+    
+    /* did we not find the token? */
+    if (def_p->de_string == NULL) {
+      (void)fprintf(stderr, "%s: could not find tag '%s' in '%s'\n",
+		    argv_program, tag_find, path_p);
+      exit(1);
     }
   }
   
   return new_debug;
 }
 
+/*
+ * List the tags that in the files.
+ */
+static	void	list_tags(void)
+{
+  char		path[1024], *path_p, token[64];
+  default_t	*def_p;
+  const char	*home_p;
+  long		new_debug = 0;
+  FILE		*rc_file;
+  
+  /* do we need to have a home variable? */
+  if (inpath == NULL) {
+    
+    /* first we try to read the RC file from the current directory */
+    rc_file = fopen(DEFAULT_CONFIG, "r");
+    if (rc_file == NULL) {
+      
+      /* if no file in current directory, try home directory */
+      home_p = getenv(HOME_ENVIRON);
+      if (home_p == NULL) {
+	(void)fprintf(stderr, "%s: could not find variable '%s'\n",
+		      argv_program, HOME_ENVIRON);
+	exit(1);
+      }
+      
+      (void)loc_snprintf(path, sizeof(path), "%s/%s", home_p, DEFAULT_CONFIG);
+      path_p = path;
+      
+      rc_file = fopen(path, "r");
+      /* we don't check for error right here */
+    }
+    else {
+      path_p = DEFAULT_CONFIG;
+    }
+  }
+  else {
+    
+    /* open the specified file */
+    rc_file = fopen(inpath, "r");
+    /* we assume that if the file was specified, it must be there */
+    if (rc_file == NULL) {
+      (void)fprintf(stderr, "%s: could not read '%s': ",
+		    argv_program, inpath);
+      perror("");
+      exit(1);
+    }
+    path_p = inpath;
+  }
+  
+  if (rc_file != NULL) {
+    (void)fprintf(stderr, "Tags available from '%s':\n", path_p);
+    
+    while (read_next_token(rc_file, &new_debug, token, sizeof(token)) == 1) {
+      if (verbose_b) {
+	(void)fprintf(stderr, "%s (%#lx):\n", token, new_debug);
+	dump_debug(new_debug);
+      }
+      else {
+	(void)fprintf(stderr, "%s\n", token);
+      }
+    }
+    
+    (void)fclose(rc_file);
+  }
+  
+  (void)fprintf(stderr, "\n");
+  (void)fprintf(stderr, "Tags available by default:\n");
+  
+  for (def_p = defaults; def_p->de_string != NULL; def_p++) {
+    if (verbose_b) {
+      (void)fprintf(stderr, "%s (%#lx):\n",
+		    def_p->de_string, def_p->de_flags);
+      dump_debug(def_p->de_flags);
+    }
+    else {
+      (void)fprintf(stderr, "%s\n", def_p->de_string);
+    }
+  }
+}
+
 /*
  * dump the current settings of the malloc variables
  */
 static	void	dump_current(void)
 {
-  char		*tok_p;
-  char		*lpath, *start_file;
-  unsigned long	addr;
-  int		addr_count, inter, lock_on, start_line, start_count;
-  long		flags;
+  char		*lpath, *start_file, token[64];
+  unsigned long	addr, inter;
+  long		addr_count;
+  int		lock_on, start_line, start_count;
+  unsigned int	flags;
+  
+  (void)fprintf(stderr, "Debug Malloc Utility: http://www.dmalloc.com/\n");
+  (void)fprintf(stderr,
+		"  For a list of the command-line options enter: %s --usage\n",
+		argv_argv[0]);
   
   _dmalloc_environ_get(OPTIONS_ENVIRON, &addr, &addr_count, &flags,
 		       &inter, &lock_on, &lpath,
 		       &start_file, &start_line, &start_count);
   
-  if (flags == DEBUG_INIT) {
+  if (flags == (unsigned int)DEBUG_INIT) {
     (void)fprintf(stderr, "Debug-Flags  not-set\n");
   }
   else {
-    (void)process(flags, NULL, &tok_p);
-    (void)fprintf(stderr, "Debug-Flags %#lx (%ld) (%s)\n", flags, flags,
-		  tok_p);
+    (void)process(flags, NULL, token, sizeof(token));
+    (void)fprintf(stderr, "Debug-Flags %#x (%u) (%s)\n",
+		  flags, flags, token);
     if (verbose_b) {
       dump_debug(flags);
     }
@@ -513,7 +689,7 @@ static	void	dump_current(void)
       (void)fprintf(stderr, "Address      %#lx\n", (long)addr);
     }
     else {
-      (void)fprintf(stderr, "Address      %#lx, count = %d\n",
+      (void)fprintf(stderr, "Address      %#lx, count = %ld\n",
 		    (long)addr, addr_count);
     }
   }
@@ -522,21 +698,15 @@ static	void	dump_current(void)
     (void)fprintf(stderr, "Interval     not-set\n");
   }
   else {
-    (void)fprintf(stderr, "Interval     %d\n", inter);
+    (void)fprintf(stderr, "Interval     %lu\n", inter);
   }
   
-#if LOCK_THREADS
   if (lock_on == LOCK_ON_INIT) {
     (void)fprintf(stderr, "Lock-On      not-set\n");
   }
   else {
     (void)fprintf(stderr, "Lock-On      %d\n", lock_on);
   }
-#else
-  if (very_verbose_b) {
-    (void)fprintf(stderr, "Lock-On      not-configured\n");
-  }
-#endif
   
   if (lpath == LOGPATH_INIT) {
     (void)fprintf(stderr, "Logpath      not-set\n");
@@ -565,16 +735,20 @@ static	void    set_variable(const char *var, const char *value)
   char	comm[1024];
   
   if (bourne_b) {
-    (void)sprintf(comm, "%s=%s;\nexport %s;\n", var, value, var);
+    (void)loc_snprintf(comm, sizeof(comm), "%s=%s;\nexport %s;\n",
+		       var, value, var);
+  }
+  else if (gdb_b) {
+    (void)loc_snprintf(comm, sizeof(comm), "set env %s %s;\n", var, value);
   }
   else {
-    (void)sprintf(comm, "setenv %s %s;\n", var, value);
+    (void)loc_snprintf(comm, sizeof(comm), "setenv %s %s;\n", var, value);
   }
   
-  if (! no_changes_b) {
+  if (make_changes_b) {
     (void)printf("%s", comm);
   }
-  if (no_changes_b || verbose_b) {
+  if ((! make_changes_b) || verbose_b) {
     (void)fprintf(stderr, "Outputed:\n");
     (void)fprintf(stderr, "%s", comm);
   }
@@ -601,11 +775,11 @@ int	main(int argc, char **argv)
   char		buf[1024];
   int		debug_set_b = FALSE, set_b = FALSE;
   char		*lpath = LOGPATH_INIT, *sfile = START_FILE_INIT;
-  unsigned long	addr = ADDRESS_INIT;
-  int		addr_count = ADDRESS_COUNT_INIT, inter = INTERVAL_INIT;
+  unsigned long	addr = ADDRESS_INIT, inter = INTERVAL_INIT;
+  long		addr_count = ADDRESS_COUNT_INIT;
   int		lock_on = LOCK_ON_INIT;
   int		sline = START_LINE_INIT, scount = START_COUNT_INIT;
-  long		flags = DEBUG_INIT;
+  unsigned int	flags = DEBUG_INIT;
   
   argv_help_string = "Sets dmalloc library env variables.  Also try --usage.";
   argv_version_string = dmalloc_version;
@@ -617,7 +791,7 @@ int	main(int argc, char **argv)
   }
   
   /* try to figure out the shell we are using */
-  if (! bourne_b && ! cshell_b) {
+  if ((! bourne_b) && (! cshell_b) && (! gdb_b)) {
     choose_shell();
   }
   
@@ -631,16 +805,16 @@ int	main(int argc, char **argv)
       (void)fprintf(stderr, "%s: warning -d ignored, processing tag '%s'\n",
 		    argv_program, tag);
     }
-    debug = process(0L, tag, NULL);
+    debug = process(0L, tag, NULL, 0);
     debug_set_b = TRUE;
   }
   
   if (plus.aa_entry_n > 0) {
-    int		plusc;
+    int		plus_c;
     
     /* get current debug value and add tokens if possible */
     if (debug == NO_VALUE) {
-      if (flags == DEBUG_INIT) {
+      if (flags == (unsigned int)DEBUG_INIT) {
 	debug = 0;
       }
       else {
@@ -648,17 +822,17 @@ int	main(int argc, char **argv)
       }
     }
     
-    for (plusc = 0; plusc < plus.aa_entry_n; plusc++) {
-      debug |= token_to_value(ARGV_ARRAY_ENTRY(plus, char *, plusc));
+    for (plus_c = 0; plus_c < plus.aa_entry_n; plus_c++) {
+      debug |= token_to_value(ARGV_ARRAY_ENTRY(plus, char *, plus_c));
     }
   }
   
   if (minus.aa_entry_n > 0) {
-    int		minusc;
+    int		minus_c;
     
     /* get current debug value and add tokens if possible */
     if (debug == NO_VALUE) {
-      if (flags == DEBUG_INIT) {
+      if (flags == (unsigned int)DEBUG_INIT) {
 	debug = 0;
       }
       else {
@@ -666,8 +840,8 @@ int	main(int argc, char **argv)
       }
     }
     
-    for (minusc = 0; minusc < minus.aa_entry_n; minusc++) {
-      debug &= ~token_to_value(ARGV_ARRAY_ENTRY(minus, char *, minusc));
+    for (minus_c = 0; minus_c < minus.aa_entry_n; minus_c++) {
+      debug &= ~token_to_value(ARGV_ARRAY_ENTRY(minus, char *, minus_c));
     }
   }
   
@@ -701,7 +875,7 @@ int	main(int argc, char **argv)
     addr = ADDRESS_INIT;
   }
   
-  if (interval != NO_VALUE) {
+  if (interval != INTERVAL_NO_VALUE) {
     /* NOTE: special case, == 0 causes it to be undef'ed */
     if (interval == 0) {
       inter = INTERVAL_INIT;
@@ -719,7 +893,6 @@ int	main(int argc, char **argv)
    * NOTE: this should be after the debug setting which this tests.
    */
   if (thread_lock_on != NO_VALUE) {
-#if LOCK_THREADS
     /* NOTE: special case, == 0 causes it to be undef'ed */
     if (thread_lock_on == 0) {
       lock_on = LOCK_ON_INIT;
@@ -728,14 +901,10 @@ int	main(int argc, char **argv)
       lock_on = thread_lock_on;
     }
     set_b = TRUE;
-    if (! (flags & DEBUG_ALLOW_NONLINEAR)) {
+    if (! BIT_IS_SET(flags, DEBUG_ALLOW_NONLINEAR)) {
       (void)fprintf(stderr,
 		    "WARNING: the allow-nonlinear flag is not enabled\n");
     }
-#else
-    (void)fprintf(stderr, "WARNING: LOCK_THREADS not-configured\n");
-    lock_on = LOCK_ON_INIT;
-#endif
   }
   else if (clear_b) {
     lock_on = LOCK_ON_INIT;
@@ -765,31 +934,43 @@ int	main(int argc, char **argv)
   }
   
   if (list_tags_b) {
-    process(0L, NULL, NULL);
+    list_tags();
   }
   
   if (debug_tokens_b) {
-    attr_t	*attrp;
+    attr_t		*attr_p;
+    unsigned int	left = 0x7fffffff;
+    
     (void)fprintf(stderr, "Debug Tokens:\n");
-    for (attrp = attributes; attrp->at_string != NULL; attrp++) {
+    for (attr_p = attributes; attr_p->at_string != NULL; attr_p++) {
+      /* skip any disabled tokens */
+      if (attr_p->at_value == 0 && strcmp(attr_p->at_string, "none") != 0) {
+	continue;
+      }
+      if (attr_p->at_value != 0 && (! BIT_IS_SET(left, attr_p->at_value))) {
+	/* skip any tokens we've seen before */
+	continue;
+      }
       if (very_verbose_b) {
 	(void)fprintf(stderr, "%s (%s) -- %s (%#lx)\n",
-		      attrp->at_string, attrp->at_short, attrp->at_desc,
-		      attrp->at_value);
+		      attr_p->at_string, attr_p->at_short, attr_p->at_desc,
+		      attr_p->at_value);
       }
       else if (verbose_b) {
 	(void)fprintf(stderr, "%s -- %s\n",
-		      attrp->at_string, attrp->at_desc);
+		      attr_p->at_string, attr_p->at_desc);
       }
       else {
-	(void)fprintf(stderr, "%s\n", attrp->at_string);
+	(void)fprintf(stderr, "%s\n", attr_p->at_string);
       }
+      BIT_CLEAR(left, attr_p->at_value);
     }
   }
   
   if (set_b) {
-    _dmalloc_environ_set(buf, long_tokens_b, short_tokens_b, addr, addr_count,
-			 flags, inter, lock_on, lpath, sfile, sline, scount);
+    _dmalloc_environ_set(buf, sizeof(buf), long_tokens_b, short_tokens_b,
+			 addr, addr_count, flags, inter, lock_on, lpath,
+			 sfile, sline, scount);
     set_variable(OPTIONS_ENVIRON, buf);
   }
   else if (errno_to_print == NO_VALUE
