@@ -1,27 +1,24 @@
 /*
- * local definitions for the user allocation level
+ * Local definitions for the user allocation level
  *
- * Copyright 1995 by Gray Watson
+ * Copyright 1999 by Gray Watson
  *
  * This file is part of the dmalloc package.
  *
- * Permission to use, copy, modify, and distribute this software for any
- * NON-COMMERCIAL purpose and without fee is hereby granted, provided
- * that the above copyright notice and this permission notice appear
- * in all copies, and that the name of Gray Watson not be used in
- * advertising or publicity pertaining to distribution of the document
- * or software without specific, written prior permission.
- *
- * Please see the PERMISSIONS file or contact the author for information
- * about commercial licenses.
+ * Permission to use, copy, modify, and distribute this software for
+ * any purpose and without fee is hereby granted, provided that the
+ * above copyright notice and this permission notice appear in all
+ * copies, and that the name of Gray Watson not be used in advertising
+ * or publicity pertaining to distribution of the document or software
+ * without specific, written prior permission.
  *
  * Gray Watson makes no representations about the suitability of the
  * software described herein for any purpose.  It is provided "as is"
  * without express or implied warranty.
  *
- * The author may be contacted at gray.watson@letters.com
+ * The author may be contacted via http://www.dmalloc.com/
  *
- * $Id: dmalloc_loc.h,v 1.37 1997/12/05 21:09:42 gray Exp $
+ * $Id: dmalloc_loc.h,v 1.42 1999/03/08 04:59:18 gray Exp $
  */
 
 #ifndef __DMALLOC_LOC_H__
@@ -34,7 +31,10 @@
  */
 #define OPTIONS_ENVIRON		"DMALLOC_OPTIONS"
 
-/******************************* useful defines ******************************/
+/*
+ * web home directory
+ */
+#define DMALLOC_HOME		"http://www.dmalloc.com/"
 
 /*
  * standard int return codes
@@ -72,7 +72,13 @@
 #define	STDERR		2		/* fileno(stderr) */
 
 /*
- * min/max macros
+ * Default values for the file and line variables.
+ */
+#define DMALLOC_DEFAULT_FILE	0L
+#define DMALLOC_DEFAULT_LINE	0
+
+/*
+ * Min/max macros
  *
  * WARNING: these use their arguments multiple times which may be bad
  */
@@ -94,7 +100,7 @@
 #define BIT_IS_SET(v,f)		((v) & (f))
 
 /*
- * global malloc defines
+ * Global malloc defines
  */
 #define BLOCK_SIZE		(int)(1 << BASIC_BLOCK)	/* size of a block */
 
