@@ -1,25 +1,24 @@
 /*
- * test program for malloc code
+ * Test program for malloc code
  *
- * Copyright 1995 by Gray Watson
+ * Copyright 1999 by Gray Watson
  *
  * This file is part of the dmalloc package.
  *
- * Permission to use, copy, modify, and distribute this software for any
- * NON-COMMERCIAL purpose and without fee is hereby granted, provided
- * that the above copyright notice and this permission notice appear
- * in all copies, and that the name of Gray Watson not be used in
- * advertising or publicity pertaining to distribution of the document
- * or software without specific, written prior permission.
- *
- * Please see the PERMISSIONS file or contact the author for information
- * about commercial licenses.
+ * Permission to use, copy, modify, and distribute this software for
+ * any purpose and without fee is hereby granted, provided that the
+ * above copyright notice and this permission notice appear in all
+ * copies, and that the name of Gray Watson not be used in advertising
+ * or publicity pertaining to distribution of the document or software
+ * without specific, written prior permission.
  *
  * Gray Watson makes no representations about the suitability of the
  * software described herein for any purpose.  It is provided "as is"
  * without express or implied warranty.
  *
- * The author may be contacted at gray.watson@letters.com
+ * The author may be contacted via http://www.dmalloc.com/
+ *
+ * $Id: dmalloc_t.c,v 1.82 1999/03/10 23:16:13 gray Exp $
  */
 
 /*
@@ -29,9 +28,23 @@
 
 #include <stdio.h>				/* for stdin */
 
+#if HAVE_STDLIB_H
+# include <stdlib.h>				/* for atoi + */
+#endif
 #if HAVE_STRING_H
 # include <string.h>
 #endif
+#if HAVE_UNISTD_H
+# include <unistd.h>
+#endif
+
+#include "conf.h"
+
+#if HAVE_TIME
+# ifdef TIME_INCLUDE
+#  include TIME_INCLUDE
+# endif
+#endif
 
 #include "dmalloc_argv.h"
 #include "dmalloc.h"
@@ -39,18 +52,16 @@
 /*
  * NOTE: these are only needed to test certain features of the library.
  */
-#include "conf.h"
 #include "debug_val.h"
 #include "error_val.h"
 
 #if INCLUDE_RCS_IDS
+#ifdef __GNUC__
+#ident "$Id: dmalloc_t.c,v 1.82 1999/03/10 23:16:13 gray Exp $";
+#else
 static	char	*rcs_id =
-  "$Id: dmalloc_t.c,v 1.63 1997/12/22 00:32:00 gray Exp $";
+  "$Id: dmalloc_t.c,v 1.82 1999/03/10 23:16:13 gray Exp $";
 #endif
-
-/* external routines */
-#if HAVE_SBRK
-extern	char		*sbrk(const int incr);	/* to extend the heap */
 #endif
 
 #define INTER_CHAR		'i'
@@ -59,8 +70,6 @@ extern	char		*sbrk(const int incr);	/* to extend the heap */
 #define MAX_ALLOC		(1024 * 1024)
 #define MIN_AVAIL		10
 
-#define RANDOM_VALUE(x)		((rand() % ((x) * 10)) / 10)
-
 /* pointer tracking structure */
 struct pnt_info_st {
   long			pi_crc;			/* crc of storage */
@@ -75,34 +84,37 @@ static	pnt_info_t	*pointer_grid;
 
 /* argument variables */
 static	int		default_iter_n = DEFAULT_ITERATIONS; /* # of iters */
-static	char		interactive_b = ARGV_FALSE;	/* interactive flag */
-static	char		no_special_b = ARGV_FALSE;	/* no-special flag */
+static	int		interactive_b = ARGV_FALSE;	/* interactive flag */
+static	int		log_trans_b = ARGV_FALSE;	/* log transactions */
+static	int		no_special_b = ARGV_FALSE;	/* no-special flag */
 static	int		max_alloc = MAX_ALLOC;		/* amt of mem to use */
 static	int		max_pointers = MAX_POINTERS;	/* # of pnts to use */
-static	char		random_debug_b = ARGV_FALSE;	/* random flag */
-static	char		silent_b = ARGV_FALSE;		/* silent flag */
+static	int		random_debug_b = ARGV_FALSE;	/* random flag */
+static	int		silent_b = ARGV_FALSE;		/* silent flag */
 static	unsigned int	seed_random = 0;		/* random seed */
-static	char		verbose_b = ARGV_FALSE;		/* verbose flag */
+static	int		verbose_b = ARGV_FALSE;		/* verbose flag */
 
 static	argv_t		arg_list[] = {
-  { INTER_CHAR,	"interactive",		ARGV_BOOL,		&interactive_b,
+  { INTER_CHAR,	"interactive",		ARGV_BOOL_INT,		&interactive_b,
       NULL,			"turn on interactive mode" },
+  { 'l',	"log-trans",		ARGV_BOOL_INT,		&log_trans_b,
+      NULL,			"log transactions via tracking-func" },
   { 'm',	"max-alloc",		ARGV_INT,		&max_alloc,
       "bytes",			"maximum allocation to test" },
-  { 'n',	"no-special",		ARGV_BOOL,		&no_special_b,
+  { 'n',	"no-special",		ARGV_BOOL_INT,		&no_special_b,
       NULL,			"do not run special tests" },
   { 'p',	"max-pointers",		ARGV_INT,		&max_pointers,
       "pointers",		"number of pointers to test" },
-  { 'r',	"random-debug",		ARGV_BOOL,	       &random_debug_b,
+  { 'r',	"random-debug",		ARGV_BOOL_INT,	       &random_debug_b,
       NULL,			"randomly change debug flag" },
-  { 's',	"silent",		ARGV_BOOL,		&silent_b,
+  { 's',	"silent",		ARGV_BOOL_INT,		&silent_b,
       NULL,			"do not display messages" },
   { 'S',	"seed-random",		ARGV_U_INT,		&seed_random,
       "number",			"seed for random function" },
-  { 't',	"times",		ARGV_INT,		&default_iter_n,
+  { 't',	"times",		ARGV_INT,	       &default_iter_n,
       "number",			"number of iterations to run" },
-  { 'v',	"verbose",		ARGV_BOOL,		&verbose_b,
-      NULL,			"enables verbose messages" },
+  { 'v',	"verbose",		ARGV_BOOL_INT,		&verbose_b,
+    NULL,			"enables verbose messages" },
   { ARGV_LAST }
 };
 
@@ -160,17 +172,34 @@ static	void	*get_address(void)
   return pnt;
 }
 
+/*
+ * Select a random value in VAL
+ */
+static	int	random_value(const int val)
+{
+  int	ret;
+  
+#if HAVE_RANDOM
+  ret = ((random() % (val * 10)) / 10);
+#else
+  ret = ((rand() % (val * 10)) / 10);
+#endif
+  
+  return ret;
+}
+
 /*
  * Try ITER_N random program iterations, returns 1 on success else 0
  */
 static	int	do_random(const int iter_n)
 {
   int		iter_c, last, amount, max = max_alloc, flags, free_c;
+  char		*chunk_p;
   pnt_info_t	*free_p, *used_p = NULL;
   pnt_info_t	*pnt_p, *last_p, *this_p;
   
   dmalloc_errno = ERROR_NONE;
-  last = 0;
+  last = ERROR_NONE;
   flags = dmalloc_debug_current();
   
   pointer_grid = (pnt_info_t *)malloc(sizeof(pnt_info_t) * max_pointers);
@@ -201,7 +230,7 @@ static	int	do_random(const int iter_n)
     }
     
     if (random_debug_b) {
-      which = RANDOM_VALUE(sizeof(int) * 8);
+      which = random_value(sizeof(int) * 8);
       flags ^= 1 << which;
       if (verbose_b) {
 	(void)printf("%d: debug flags = %#x\n", iter_c + 1, flags);
@@ -221,7 +250,7 @@ static	int	do_random(const int iter_n)
     }
     
     /* decide whether to malloc a new pointer or free/realloc an existing */
-    which = RANDOM_VALUE(4);
+    which = random_value(4);
     
     /*
      * < MIN_AVAIL means alloc as long as we have enough memory and
@@ -229,7 +258,7 @@ static	int	do_random(const int iter_n)
      */
     if (free_c == max_pointers
 	|| (free_c > 0 && which < 3 && max >= MIN_AVAIL)) {
-      amount = RANDOM_VALUE(max / 2);
+      amount = random_value(max / 2);
 #if ALLOW_ALLOC_ZERO_SIZE == 0
       if (amount == 0) {
 	amount = 1;
@@ -237,26 +266,15 @@ static	int	do_random(const int iter_n)
 #endif
       
       if (flags & DEBUG_ALLOW_NONLINEAR) {
-	which = RANDOM_VALUE(10);
+	which = random_value(16);
       }
       else {
-	which = RANDOM_VALUE(9);
+	which = random_value(15);
       }
       
       switch (which) {
-
-      case 0: case 1: case 2:
-	pnt_p = free_p;
-	pnt_p->pi_pnt = calloc(amount, sizeof(char));
 	
-	if (verbose_b) {
-	  (void)printf("%d: calloc %d of max %d into slot %d.  got %#lx\n",
-		       iter_c + 1, amount, max, pnt_p - pointer_grid,
-		       (long)pnt_p->pi_pnt);
-	}
-	break;
-
-      case 3: case 4: case 5:
+      case 0: case 1: case 2:
 	pnt_p = free_p;
 	pnt_p->pi_pnt = malloc(amount);
 	
@@ -267,12 +285,38 @@ static	int	do_random(const int iter_n)
 	}
 	break;
 	
+      case 3: case 4: case 5:
+	pnt_p = free_p;
+	pnt_p->pi_pnt = calloc(amount, sizeof(char));
+	
+	if (verbose_b) {
+	  (void)printf("%d: calloc %d of max %d into slot %d.  got %#lx\n",
+		       iter_c + 1, amount, max, pnt_p - pointer_grid,
+		       (long)pnt_p->pi_pnt);
+	}
+	
+	/* test the returned block to make sure that is has been cleared */
+	if (pnt_p->pi_pnt != NULL) {
+	  for (chunk_p = pnt_p->pi_pnt;
+	       chunk_p < (char *)pnt_p->pi_pnt + amount;
+	       chunk_p++) {
+	    if (*chunk_p != '\0') {
+	      if (! silent_b) {
+		(void)printf("calloc of %d was not fully zeroed on iteration #%d\n",
+			     amount, iter_c + 1);
+	      }
+	      break;
+	    }
+	  }
+	}
+	break;
+
       case 6: case 7: case 8:
 	if (free_c == max_pointers) {
 	  continue;
 	}
 	
-	which = RANDOM_VALUE(max_pointers - free_c);
+	which = random_value(max_pointers - free_c);
 	for (pnt_p = used_p; which > 0; which--) {
 	  pnt_p = pnt_p->pi_next;
 	}
@@ -282,12 +326,58 @@ static	int	do_random(const int iter_n)
 	
 	if (verbose_b) {
 	  (void)printf("%d: realloc %d from %d of max %d slot %d.  got %#lx\n",
-		       iter_c + 1, pnt_p->pi_size, amount, max,
+		       iter_c + 1, amount, pnt_p->pi_size, max,
 		       pnt_p - pointer_grid, (long)pnt_p->pi_pnt);
 	}
 	break;
 	
-      default:
+      case 9: case 10: case 11:
+	if (free_c == max_pointers) {
+	  continue;
+	}
+	
+	which = random_value(max_pointers - free_c);
+	for (pnt_p = used_p; which > 0; which--) {
+	  pnt_p = pnt_p->pi_next;
+	}
+	
+	pnt_p->pi_pnt = recalloc(pnt_p->pi_pnt, amount);
+	max += pnt_p->pi_size;
+	
+	if (verbose_b) {
+	  (void)printf("%d: recalloc %d from %d of max %d slot %d.  got %#lx\n",
+		       iter_c + 1, amount, pnt_p->pi_size, max,
+		       pnt_p - pointer_grid, (long)pnt_p->pi_pnt);
+	}
+	
+	/* test the returned block to make sure that is has been cleared */
+	if (pnt_p->pi_pnt != NULL && amount > pnt_p->pi_size) {
+	  for (chunk_p = (char *)pnt_p->pi_pnt + pnt_p->pi_size;
+	       chunk_p < (char *)pnt_p->pi_pnt + amount;
+	       chunk_p++) {
+	    if (*chunk_p != '\0') {
+	      if (! silent_b) {
+		(void)printf("recalloc %d from %d was not fully zeroed on iteration #%d\n",
+			     amount, pnt_p->pi_size, iter_c + 1);
+	      }
+	      break;
+	    }
+	  }
+	}
+	break;
+	
+      case 12: case 13: case 14:
+	pnt_p = free_p;
+	pnt_p->pi_pnt = valloc(amount);
+	
+	if (verbose_b) {
+	  (void)printf("%d: valloc %d of max %d into slot %d.  got %#lx\n",
+		       iter_c + 1, amount, max, pnt_p - pointer_grid,
+		       (long)pnt_p->pi_pnt);
+	}
+	break;
+	
+      case 15:
 #if HAVE_SBRK
 	{
 	  void	*mem;
@@ -300,10 +390,10 @@ static	int	do_random(const int iter_n)
 	  pnt_p = NULL;
 	}
 	break;
-#else
-	iter_c++;
-	continue;
 #endif
+	
+      default:
+	break;
       }
       
       if (pnt_p != NULL) {
@@ -333,9 +423,9 @@ static	int	do_random(const int iter_n)
     }
     
     /*
-     * choose a rand slot to free and make sure it is not a free-slot
+     * choose a random slot to free and make sure it is not a free-slot
      */
-    which = RANDOM_VALUE(max_pointers - free_c);
+    which = random_value(max_pointers - free_c);
     for (pnt_p = used_p; which > 0; which--) {
       pnt_p = pnt_p->pi_next;
     }
@@ -351,7 +441,8 @@ static	int	do_random(const int iter_n)
     pnt_p->pi_pnt = NULL;
     
     /* find pnt in the used list */
-    for (this_p = used_p, last_p = NULL; this_p != NULL;
+    for (this_p = used_p, last_p = NULL;
+	 this_p != NULL;
 	 last_p = this_p, this_p = this_p->pi_next) {
       if (this_p == pnt_p) {
 	break;
@@ -401,6 +492,8 @@ static	int	check_special(void)
     (void)printf("The following tests will generate errors:\n");
   }
   
+  /********************/
+
   if (! silent_b) {
     (void)printf("  Trying to realloc a 0L pointer.\n");
   }
@@ -426,6 +519,8 @@ static	int	check_special(void)
   }
 #endif
   
+  /********************/
+
   if (! silent_b) {
     (void)printf("  Trying to free 0L pointer.\n");
   }
@@ -443,6 +538,8 @@ static	int	check_special(void)
   }
 #endif
   
+  /********************/
+
   if (! silent_b) {
     (void)printf("  Allocating a block of too-many bytes.\n");
   }
@@ -457,6 +554,8 @@ static	int	check_special(void)
     free(pnt);
   }
   
+  /********************/
+  
   if (dmalloc_errno == ERROR_NONE) {
     ret = 1;
   }
@@ -504,9 +603,9 @@ static	void	do_interactive(void)
       (void)printf("\tmalloc    - allocate memory\n");
       (void)printf("\tcalloc    - allocate/clear memory\n");
       (void)printf("\trealloc   - reallocate memory\n");
-#if HAVE_STRDUP
+      (void)printf("\tmemalign  - allocate aligned memory\n");
+      (void)printf("\tvalloc    - allocate page-aligned memory\n");
       (void)printf("\tstrdup    - allocate a string\n");
-#endif
       (void)printf("\tfree      - deallocate memory\n\n");
       
       (void)printf("\tmap       - map the heap to the logfile\n");
@@ -572,7 +671,53 @@ static	void	do_interactive(void)
       continue;
     }
     
-#if HAVE_STRDUP
+    if (strncmp(line, "recalloc", len) == 0) {
+      int	size;
+      
+      pnt = get_address();
+      
+      (void)printf("How much to recalloc: ");
+      if (fgets(line, sizeof(line), stdin) == NULL) {
+	break;
+      }
+      size = atoi(line);
+      
+      (void)printf("realloc(%#lx, %d) returned '%#lx'\n",
+		   (long)pnt, size, (long)recalloc(pnt, size));
+      
+      continue;
+    }
+    
+    if (strncmp(line, "memalign", len) == 0) {
+      int	alignment, size;
+      
+      (void)printf("Alignment in bytes: ");
+      if (fgets(line, sizeof(line), stdin) == NULL) {
+	break;
+      }
+      alignment = atoi(line);
+      (void)printf("How much to memalign: ");
+      if (fgets(line, sizeof(line), stdin) == NULL) {
+	break;
+      }
+      size = atoi(line);
+      (void)printf("memalign(%d, %d) returned '%#lx'\n",
+		   alignment, size, (long)memalign(alignment, size));
+      continue;
+    }
+    
+    if (strncmp(line, "valloc", len) == 0) {
+      int	size;
+      
+      (void)printf("How much to valloc: ");
+      if (fgets(line, sizeof(line), stdin) == NULL) {
+	break;
+      }
+      size = atoi(line);
+      (void)printf("valloc(%d) returned '%#lx'\n", size, (long)valloc(size));
+      continue;
+    }
+    
     if (strncmp(line, "strdup", len) == 0) {
       (void)printf("Enter a string to strdup: ");
       if (fgets(line, sizeof(line), stdin) == NULL) {
@@ -581,7 +726,6 @@ static	void	do_interactive(void)
       (void)printf("strdup returned '%#lx'\n", (long)strdup(line));
       continue;
     }
-#endif
     
     if (strncmp(line, "free", len) == 0) {
       pnt = get_address();
@@ -684,6 +828,74 @@ static	void	do_interactive(void)
   }
 }
 
+/*
+ * Allocation tracking function called each time an allocation occurs.
+ * FILE may be a return address if LINE is 0.  FUNC_ID is one of the
+ * above DMALLOC_FUNC_ defines.  BYTE_SIZE is how many bytes were
+ * requested with a possible ALIGNMENT.  OLD_ADDR is for realloc and
+ * free functions.  NEW_ADDR is the pointer returned by the allocation
+ * functions.
+ */
+static	void	track_alloc_trxn(const char *file, const unsigned int line,
+				 const int func_id,
+				 const DMALLOC_SIZE byte_size,
+				 const DMALLOC_SIZE alignment,
+				 const DMALLOC_PNT old_addr,
+				 const DMALLOC_PNT new_addr)
+{
+  char	file_line[64];
+  
+  if (file == NULL && line == 0) {
+    strcpy(file_line, "unknown");
+  }
+  else if (line == 0) {
+    (void)sprintf(file_line, "ra=%#lx", (long)file);
+  }
+  else {
+    (void)sprintf(file_line, "%s:%d", file, line);
+  }
+  
+  switch (func_id) {
+  case DMALLOC_FUNC_MALLOC:
+    (void)printf("%s malloc %d bytes got %#lx\n",
+		 file_line, byte_size, (long)new_addr);
+    break;
+  case DMALLOC_FUNC_CALLOC:
+    (void)printf("%s calloc %d bytes got %#lx\n",
+		 file_line, byte_size, (long)new_addr);
+    break;
+  case DMALLOC_FUNC_REALLOC:
+    (void)printf("%s realloc %d bytes from %#lx got %#lx\n",
+		 file_line, byte_size, (long)old_addr, (long)new_addr);
+    break;
+  case DMALLOC_FUNC_RECALLOC:
+    (void)printf("%s recalloc %d bytes from %#lx got %#lx\n",
+		 file_line, byte_size, (long)old_addr, (long)new_addr);
+    break;
+  case DMALLOC_FUNC_MEMALIGN:
+    (void)printf("%s memalign %d bytes alignment %d got %#lx\n",
+		 file_line, byte_size, alignment, (long)new_addr);
+    break;
+  case DMALLOC_FUNC_VALLOC:
+    (void)printf("%s valloc %d bytes alignment %d got %#lx\n",
+		 file_line, byte_size, alignment, (long)new_addr);
+    break;
+  case DMALLOC_FUNC_STRDUP:
+    (void)printf("%s strdup %d bytes ot %#lx\n",
+		 file_line, byte_size, (long)new_addr);
+    break;
+  case DMALLOC_FUNC_FREE:
+    (void)printf("%s free %#lx\n", file_line, (long)old_addr);
+    break;
+  default:
+    (void)printf("%s unknown function %d bytes, %d alignment, %#lx old-addr "
+		 "%#lx new-addr\n",
+		 file_line, byte_size, alignment, (long)old_addr,
+		 (long)new_addr);
+    break;
+  }
+}
+
 int	main(int argc, char **argv)
 {
   int	ret;
@@ -696,12 +908,35 @@ int	main(int argc, char **argv)
   
   /* repeat until we get a non 0 seed */
   while (seed_random == 0) {
-    seed_random = time(0) ^ 0xDEABEEF;
+#ifdef HAVE_TIME
+#ifdef HAVE_GETPID
+    seed_random = time(0) ^ getpid();
+#else /* ! HAVE_GETPID */
+    seed_random = time(0) ^ 0xDEADBEEF;
+#endif /* ! HAVE_GETPID */
+#else /* ! HAVE_TIME */
+#ifdef HAVE_GETPID
+    seed_random = getpid();
+#else /* ! HAVE_GETPID */
+    /* okay, I give up */
+    seed_random = 0xDEADBEEF;
+#endif /* ! HAVE_GETPID */
+#endif /* ! HAVE_TIME */
   }
+#if HAVE_RANDOM
+  (void)srandom(seed_random);
+#else
   (void)srand(seed_random);
+#endif
   
   if (! silent_b) {
-    (void)printf("Random seed = %u\n", seed_random);
+    (void)printf("Random seed is %u\n", seed_random);
+  }
+  
+  dmalloc_message("random seed is %u\n", seed_random);
+  
+  if (log_trans_b) {
+    dmalloc_track(track_alloc_trxn);
   }
   
   if (interactive_b) {
@@ -720,9 +955,27 @@ int	main(int argc, char **argv)
     }
   }
   
-  /* don't allow silent dumps */
-  if (! no_special_b
-      && (! (silent_b && (dmalloc_debug_current() & DEBUG_ERROR_ABORT)))) {
+  if (dmalloc_errno != ERROR_NONE) {
+    /*
+     * Even if we are silent, we must give the random seed which is
+     * the only way we can reproduce the problem.
+     */
+    if (silent_b) {
+      (void)fprintf(stderr, "Random seed is %u.  Error: %s (%d)\n",
+		    seed_random, dmalloc_strerror(dmalloc_errno),
+		    dmalloc_errno);
+    }
+    else {
+      (void)fprintf(stderr, "Final malloc error: %s (%d)\n",
+		    dmalloc_strerror(dmalloc_errno), dmalloc_errno);
+    }
+    exit(1);
+  }
+  
+  /* check the special malloc functions but don't allow silent dumps */
+  if ((! no_special_b)
+      && (! (silent_b
+	     && (dmalloc_debug_current() & DEBUG_ERROR_ABORT)))) {
     if (! silent_b) {
       (void)printf("Running special tests...\n");
     }
@@ -732,24 +985,20 @@ int	main(int argc, char **argv)
     }
   }
   
+  argv_cleanup(arg_list);
+  
+  /* last thing is to verify the heap */
+  ret = malloc_verify(NULL);
+  if (ret != DMALLOC_VERIFY_NOERROR) {
+    (void)fprintf(stderr, "Final malloc_verify returned failure: %s (%d)\n",
+		  dmalloc_strerror(dmalloc_errno), dmalloc_errno);
+  }
+  
   /* you will need this if you can't auto-shutdown */
 #if HAVE_ATEXIT == 0 && HAVE_ON_EXIT == 0 && FINI_DMALLOC == 0
   /* shutdown the alloc routines */
   malloc_shutdown();
 #endif
   
-  argv_cleanup(arg_list);
-  
-  ret = malloc_verify(NULL);
-  if (! silent_b) {
-    (void)printf("Final malloc_verify returned '%s'\n",
-		 (ret == DMALLOC_VERIFY_NOERROR ? "success" : "failure"));
-  }
-  
-  if (dmalloc_errno == ERROR_NONE) {
-    exit(0);
-  }
-  else {
-    exit(1);
-  }
+  exit(0);
 }
