@@ -1,25 +1,24 @@
 /*
- * environment handling routines
+ * Environment handling routines
  *
- * Copyright 1995 by Gray Watson
+ * Copyright 1999 by Gray Watson
  *
  * This file is part of the dmalloc package.
  *
- * Permission to use, copy, modify, and distribute this software for any
- * NON-COMMERCIAL purpose and without fee is hereby granted, provided
- * that the above copyright notice and this permission notice appear
- * in all copies, and that the name of Gray Watson not be used in
- * advertising or publicity pertaining to distribution of the document
- * or software without specific, written prior permission.
- *
- * Please see the PERMISSIONS file or contact the author for information
- * about commercial licenses.
+ * Permission to use, copy, modify, and distribute this software for
+ * any purpose and without fee is hereby granted, provided that the
+ * above copyright notice and this permission notice appear in all
+ * copies, and that the name of Gray Watson not be used in advertising
+ * or publicity pertaining to distribution of the document or software
+ * without specific, written prior permission.
  *
  * Gray Watson makes no representations about the suitability of the
  * software described herein for any purpose.  It is provided "as is"
  * without express or implied warranty.
  *
- * The author may be contacted at gray.watson@letters.com
+ * The author may be contacted via http://www.dmalloc.com/
+ *
+ * $Id: env.c,v 1.22 1999/03/10 16:41:16 gray Exp $
  */
 
 /*
@@ -35,18 +34,26 @@
 #if HAVE_STRING_H
 # include <string.h>
 #endif
+#if HAVE_UNISTD_H
+# include <unistd.h>				/* for getpid */
+#endif
 
 #include "conf.h"
 #include "dmalloc.h"
 
+#include "compat.h"
 #include "dmalloc_loc.h"
 #include "debug_tok.h"
 #include "env.h"
 #include "error.h"
 
 #if INCLUDE_RCS_IDS
+#ifdef __GNUC__
+#ident "$Id: env.c,v 1.22 1999/03/10 16:41:16 gray Exp $";
+#else
 static	char	*rcs_id =
-  "$Id: env.c,v 1.13 1997/12/22 00:32:01 gray Exp $";
+  "$Id: env.c,v 1.22 1999/03/10 16:41:16 gray Exp $";
+#endif
 #endif
 
 /* local variables */
@@ -63,7 +70,8 @@ static	long	hex_to_long(const char *str)
   long		ret;
   
   /* strip off spaces */
-  for (; *str == ' ' || *str == '\t'; str++);
+  for (; *str == ' ' || *str == '\t'; str++) {
+  }
   
   /* skip a leading 0[xX] */
   if (*str == '0' && (*(str + 1) == 'x' || *(str + 1) == 'X')) {
@@ -94,7 +102,7 @@ static	long	hex_to_long(const char *str)
  * Break up ADDR_ALL into ADDR_P and ADDR_COUNT_P
  */
 void	_dmalloc_address_break(const char *addr_all, unsigned long *addr_p,
-			       int *addr_count_p)
+			       long *addr_count_p)
 {
   char	*colon_p;
   
@@ -139,8 +147,8 @@ void	_dmalloc_start_break(const char *start_all, char **sfile_p,
  * string.
  */
 void	_dmalloc_environ_get(const char *environ, unsigned long *addr_p,
-			     int *addr_count_p, unsigned int *debug_p,
-			     int *interval_p, int *lock_on_p,
+			     long *addr_count_p, unsigned int *debug_p,
+			     unsigned long *interval_p, int *lock_on_p,
 			     char **logpath_p, char **sfile_p,
 			     int *sline_p, int *scount_p)
 {
@@ -233,7 +241,7 @@ void	_dmalloc_environ_get(const char *environ, unsigned long *addr_p,
 	&& *(this_p + len) == ASSIGNMENT_CHAR) {
       this_p += len + 1;
       if (interval_p != NULL) {
-	*interval_p = atoi(this_p);
+	*interval_p = (unsigned long)atol(this_p);
       }
       continue;
     }
@@ -258,7 +266,7 @@ void	_dmalloc_environ_get(const char *environ, unsigned long *addr_p,
        * NOTE: this may cause core dumps if this_p contains a bad
        * format string
        */
-      (void)sprintf(log_path, this_p, getpid());
+      (void)loc_snprintf(log_path, sizeof(log_path), this_p, getpid());
 #else
       (void)strcpy(log_path, this_p);
 #endif
@@ -308,15 +316,16 @@ void	_dmalloc_environ_get(const char *environ, unsigned long *addr_p,
  * Set dmalloc environ variable(s) with the values (maybe SHORT debug
  * info) into BUF.
  */
-void	_dmalloc_environ_set(char *buf, const int long_tokens_b,
+void	_dmalloc_environ_set(char *buf, const int buf_size,
+			     const int long_tokens_b,
 			     const int short_tokens_b,
 			     const unsigned long address,
-			     const int addr_count, const unsigned int debug,
+			     const long addr_count, const unsigned int debug,
 			     const int interval, const int lock_on,
 			     const char *logpath, const char *sfile,
 			     const int sline, const int scount)
 {
-  char	*buf_p = buf;
+  char	*buf_p = buf, *bounds_p = buf + buf_size;
   
   if (debug != (unsigned int)DEBUG_INIT) {
     if (short_tokens_b || long_tokens_b) {
@@ -325,64 +334,57 @@ void	_dmalloc_environ_set(char *buf, const int long_tokens_b,
       for (attr_p = attributes; attr_p->at_string != NULL; attr_p++) {
 	if (debug & attr_p->at_value) {
 	  if (short_tokens_b) {
-	    (void)sprintf(buf_p, "%s,", attr_p->at_short);
+	    buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%s,",
+				  attr_p->at_short);
 	  }
 	  else {
-	    (void)sprintf(buf_p, "%s,", attr_p->at_string);
-	  }
-	  for (; *buf_p != '\0'; buf_p++) {
+	    buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%s,",
+				  attr_p->at_string);
 	  }
 	}
       }
     }
     else {
-      (void)sprintf(buf_p, "%s%c%#x,", DEBUG_LABEL, ASSIGNMENT_CHAR, debug);
-      for (; *buf_p != '\0'; buf_p++) {
-      }
+      buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%s%c%#x,",
+			    DEBUG_LABEL, ASSIGNMENT_CHAR, debug);
     }
   }
   if (address != ADDRESS_INIT) {
     if (addr_count != ADDRESS_COUNT_INIT) {
-      (void)sprintf(buf_p, "%s%c%#lx:%d,",
-		    ADDRESS_LABEL, ASSIGNMENT_CHAR, address, addr_count);
+      buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%s%c%#lx:%ld,",
+			    ADDRESS_LABEL, ASSIGNMENT_CHAR, address,
+			    addr_count);
     }
     else {
-      (void)sprintf(buf_p, "%s%c%#lx,",
-		    ADDRESS_LABEL, ASSIGNMENT_CHAR, address);
-    }
-    for (; *buf_p != '\0'; buf_p++) {
+      buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%s%c%#lx,",
+			    ADDRESS_LABEL, ASSIGNMENT_CHAR, address);
     }
   }
   if (interval != INTERVAL_INIT) {
-    (void)sprintf(buf_p, "%s%c%d,", INTERVAL_LABEL, ASSIGNMENT_CHAR, interval);
-    for (; *buf_p != '\0'; buf_p++) {
-    }
+    buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%s%c%d,",
+			  INTERVAL_LABEL, ASSIGNMENT_CHAR, interval);
   }
   if (lock_on != LOCK_ON_INIT) {
-    (void)sprintf(buf_p, "%s%c%d,", LOCK_ON_LABEL, ASSIGNMENT_CHAR, lock_on);
-    for (; *buf_p != '\0'; buf_p++) {
-    }
+    buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%s%c%d,",
+			  LOCK_ON_LABEL, ASSIGNMENT_CHAR, lock_on);
   }
   if (logpath != LOGPATH_INIT) {
-    (void)sprintf(buf_p, "%s%c%s,", LOGFILE_LABEL, ASSIGNMENT_CHAR, logpath);
-    for (; *buf_p != '\0'; buf_p++) {
-    }
+    buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%s%c%s,",
+			  LOGFILE_LABEL, ASSIGNMENT_CHAR, logpath);
   }
   if (sfile != START_FILE_INIT) {
     if (sline != START_LINE_INIT) {
-      (void)sprintf(buf_p, "%s%c%s:%d,",
-		    START_LABEL, ASSIGNMENT_CHAR, sfile, sline);
+      buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%s%c%s:%d,",
+			    START_LABEL, ASSIGNMENT_CHAR, sfile, sline);
     }
     else {
-      (void)sprintf(buf_p, "%s%c%s,", START_LABEL, ASSIGNMENT_CHAR, sfile);
-    }
-    for (; *buf_p != '\0'; buf_p++) {
+      buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%s%c%s,",
+			    START_LABEL, ASSIGNMENT_CHAR, sfile);
     }
   }
   else if (scount != START_COUNT_INIT) {
-    (void)sprintf(buf_p, "%s%c%d,", START_LABEL, ASSIGNMENT_CHAR, scount);
-    for (; *buf_p != '\0'; buf_p++) {
-    }
+    buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%s%c%d,",
+			  START_LABEL, ASSIGNMENT_CHAR, scount);
   }
   
   /* cut off the last comma */
