@@ -1,25 +1,24 @@
 /*
  * Error and message routines
  *
- * Copyright 1995 by Gray Watson
+ * Copyright 1999 by Gray Watson
  *
  * This file is part of the dmalloc package.
  *
- * Permission to use, copy, modify, and distribute this software for any
- * NON-COMMERCIAL purpose and without fee is hereby granted, provided
- * that the above copyright notice and this permission notice appear
- * in all copies, and that the name of Gray Watson not be used in
- * advertising or publicity pertaining to distribution of the document
- * or software without specific, written prior permission.
- *
- * Please see the PERMISSIONS file or contact the author for information
- * about commercial licenses.
+ * Permission to use, copy, modify, and distribute this software for
+ * any purpose and without fee is hereby granted, provided that the
+ * above copyright notice and this permission notice appear in all
+ * copies, and that the name of Gray Watson not be used in advertising
+ * or publicity pertaining to distribution of the document or software
+ * without specific, written prior permission.
  *
  * Gray Watson makes no representations about the suitability of the
  * software described herein for any purpose.  It is provided "as is"
  * without express or implied warranty.
  *
- * The author may be contacted at gray.watson@letters.com
+ * The author may be contacted via http://www.dmalloc.com/
+ *
+ * $Id: error.c,v 1.89 1999/03/11 22:30:54 gray Exp $
  */
 
 /*
@@ -28,7 +27,20 @@
  */
 
 #include <fcntl.h>				/* for O_WRONLY, etc. */
-#include <stdarg.h>				/* for message vsprintf */
+#include <stdio.h>				/* for sprintf */
+
+#if HAVE_STDARG_H
+# include <stdarg.h>				/* for message vsprintf */
+#endif
+#if HAVE_STDLIB_H
+# include <stdlib.h>				/* for abort */
+#endif
+#if HAVE_STRING_H
+# include <string.h>				/* for strlen */
+#endif
+#if HAVE_UNISTD_H
+# include <unistd.h>				/* for write */
+#endif
 
 #include "conf.h"				/* up here for _INCLUDE */
 
@@ -39,11 +51,16 @@
 #endif
 #endif
 
-/* for timeval type -- see conf.h */
 #if STORE_TIMEVAL
-#ifdef TIMEVAL_INCLUDE
-#include TIMEVAL_INCLUDE
-#endif
+# ifdef TIMEVAL_INCLUDE
+#  include TIMEVAL_INCLUDE
+# endif
+#else
+# if HAVE_TIME
+#  ifdef TIME_INCLUDE
+#   include TIME_INCLUDE
+#  endif
+# endif
 #endif
 
 #define DMALLOC_DISABLE
@@ -59,8 +76,12 @@
 #include "version.h"
 
 #if INCLUDE_RCS_IDS
+#ifdef __GNUC__
+#ident "$Id: error.c,v 1.89 1999/03/11 22:30:54 gray Exp $";
+#else
 static	char	*rcs_id =
-  "$Id: error.c,v 1.68 1997/12/22 00:32:02 gray Exp $";
+  "$Id: error.c,v 1.89 1999/03/11 22:30:54 gray Exp $";
+#endif
 #endif
 
 #define SECS_IN_HOUR	(MINS_IN_HOUR * SECS_IN_MIN)
@@ -74,11 +95,11 @@ extern	char		*_dmalloc_strerror(const int errnum);
  * exported variables
  */
 /* logfile for dumping dmalloc info, DMALLOC_LOGFILE env var overrides this */
-char		*dmalloc_logpath = LOGPATH_INIT;
-/* internal dmalloc error number for reference purposes only */
-int		dmalloc_errno = ERROR_NONE;
+char		*_dmalloc_logpath = LOGPATH_INIT;
 /* address to look for.  when discovered call dmalloc_error() */
-DMALLOC_PNT	dmalloc_address = ADDRESS_INIT;
+DMALLOC_PNT	_dmalloc_address = ADDRESS_INIT;
+/* when to stop at an address */
+long		_dmalloc_address_seen_n = ADDRESS_COUNT_INIT;
 
 /* global debug flags that are set my DMALLOC_DEBUG environ variable */
 unsigned int	_dmalloc_flags = 0;
@@ -86,12 +107,19 @@ unsigned int	_dmalloc_flags = 0;
 /* global iteration counter for activities */
 unsigned long	_dmalloc_iter_c = 0;
 
+/* how often to check the heap */
+unsigned long	_dmalloc_check_interval = INTERVAL_INIT;
+
 #if STORE_TIMEVAL
 /* overhead information storing when the library started up for elapsed time */
-struct timeval	_dmalloc_start;
+TIMEVAL_TYPE	_dmalloc_start;
 #endif
+
+/* NOTE: we do the ifdef this way for fillproto */
 #if STORE_TIMEVAL == 0
-long		_dmalloc_start = 0;
+#if HAVE_TIME
+TIME_TYPE	_dmalloc_start = 0;
+#endif
 #endif
 
 /* global flag which indicates when we are aborting */
@@ -101,92 +129,107 @@ int		_dmalloc_aborting_b = FALSE;
 /*
  * print the time into local buffer which is returned
  */
-char	*_dmalloc_ptimeval(const struct timeval *timeval_p,
-			   const int elapsed_b)
+char	*_dmalloc_ptimeval(const TIMEVAL_TYPE *timeval_p, char *buf,
+			   const int buf_size, const int elapsed_b)
 {
-  static char	buf[64];
-  long		hrs, mins, secs;
-  long		usecs;
-  
-  buf[0] = '\0';
+  unsigned long	hrs, mins, secs;
+  unsigned long	usecs;
   
   secs = timeval_p->tv_sec;
   usecs = timeval_p->tv_usec;
   
-  if (elapsed_b || BIT_IS_SET(_dmalloc_flags, DEBUG_LOG_ELAPSED_TIME)) {
-    usecs -= _dmalloc_start.tv_usec;
-    if (usecs < 0) {
+  if (elapsed_b) {
+    if (usecs >= _dmalloc_start.tv_usec) {
+      usecs -= _dmalloc_start.tv_usec;
+    }
+    else {
+      usecs = _dmalloc_start.tv_usec - usecs;
       secs--;
-      usecs = - usecs;
     }
     secs -= _dmalloc_start.tv_sec;
+    
+    hrs = secs / SECS_IN_HOUR;
+    mins = (secs / SECS_IN_MIN) % MINS_IN_HOUR;
+    secs %= SECS_IN_MIN;
+    
+    (void)loc_snprintf(buf, buf_size, "%lu:%02lu:%02lu.%06lu",
+		       hrs, mins, secs, usecs);
+  }
+  else {
+    (void)loc_snprintf(buf, buf_size, "%lu.%06lu",
+		       secs, usecs);
   }
-  
-  hrs = secs / SECS_IN_HOUR;
-  mins = (secs / SECS_IN_MIN) % MINS_IN_HOUR;
-  secs %= SECS_IN_MIN;
-  
-  (void)sprintf(buf, "%ld:%ld:%ld.%ld", hrs, mins, secs, usecs);
   
   return buf;
 }
 #endif
 
-#if STORE_TIMEVAL == 0
+/* NOTE: we do the ifdef this way for fillproto */
+#if STORE_TIMEVAL == 0 && HAVE_TIME
 /*
  * print the time into local buffer which is returned
  */
-char	*_dmalloc_ptime(const long *time_p, const int elapsed_b)
+char	*_dmalloc_ptime(const TIME_TYPE *time_p, char *buf, const int buf_size,
+			const int elapsed_b)
 {
-  static char	buf[64];
-  long		hrs, mins, secs;
+  unsigned long	hrs, mins, secs;
   
-  buf[0] = '\0';
   secs = *time_p;
   
-  if (elapsed_b || BIT_IS_SET(_dmalloc_flags, DEBUG_LOG_ELAPSED_TIME)) {
+  if (elapsed_b) {
     secs -= _dmalloc_start;
+    
+    hrs = secs / SECS_IN_HOUR;
+    mins = (secs / SECS_IN_MIN) % MINS_IN_HOUR;
+    secs %= SECS_IN_MIN;
+    
+    (void)loc_snprintf(buf, buf_size, "%lu:%02lu:%02lu", hrs, mins, secs);
+  }
+  else {
+    (void)loc_snprintf(buf, buf_size, "%lu", secs);
   }
-  
-  hrs = secs / SECS_IN_HOUR;
-  mins = (secs / SECS_IN_MIN) % MINS_IN_HOUR;
-  secs %= SECS_IN_MIN;
-  
-  (void)sprintf(buf, "%ld:%ld:%ld", hrs, mins, secs);
   
   return buf;
 }
 #endif
 
 /*
- * message writer with printf like arguments
+ * message writer with vprintf like arguments
  */
-void	_dmalloc_message(const char *format, ...)
+void	_dmalloc_vmessage(const char *format, va_list args)
 {
   static int	outfile = -1;
-  char		str[1024], *str_p = str;
+  char		str[1024], *str_p, *bounds_p;
   int		len;
-  va_list	args;
+  
+  str_p = str;
+  bounds_p = str_p + sizeof(str);
   
   /* no logpath and no print then no workie */
-  if (dmalloc_logpath == LOGPATH_INIT
-      && ! BIT_IS_SET(_dmalloc_flags, DEBUG_PRINT_ERROR)) {
+  if (_dmalloc_logpath == LOGPATH_INIT
+      && ! BIT_IS_SET(_dmalloc_flags, DEBUG_PRINT_MESSAGES)) {
     return;
   }
   
 #if HAVE_TIME
-  /* maybe dump a time stamp */
-  if (BIT_IS_SET(_dmalloc_flags, DEBUG_LOG_STAMP)) {
-    (void)sprintf(str_p, "%ld: ", (long)time(NULL));
-    for (; *str_p != '\0'; str_p++) {
-    }
+#if LOG_TIME_NUMBER
+  str_p += loc_snprintf(str_p, bounds_p - str_p, "%lu: ",
+			(unsigned long)time(NULL));
+#endif /* LOG_TIME_NUMBER */
+#if HAVE_CTIME
+#if LOG_CTIME_STRING
+  {
+    TIME_TYPE	now;
+    now = time(NULL);
+    str_p += loc_snprintf(str_p, bounds_p - str_p, "%.24s: ", ctime(&now));
   }
-#endif
+#endif /* LOG_CTIME_STRING */
+#endif /* HAVE_CTIME */
+#endif /* HAVE_TIME */
   
 #if LOG_ITERATION_COUNT
   /* add the iteration number */
-  (void)sprintf(str_p, "%lu: ", _dmalloc_iter_c);
-  for (; *str_p != '\0'; str_p++);
+  str_p += loc_snprintf(str_p, bounds_p - str_p, "%lu: ", _dmalloc_iter_c);
 #endif
   
   /*
@@ -196,18 +239,13 @@ void	_dmalloc_message(const char *format, ...)
    */
   
   /* write the format + info into str */
-  va_start(args, format);
-  (void)vsprintf(str_p, format, args);
-  va_end(args);
+  len = loc_vsnprintf(str_p, bounds_p - str_p, format, args);
   
   /* was it an empty format? */
-  if (*str_p == '\0') {
+  if (len == 0) {
     return;
   }
-  
-  /* find the length of str */
-  for (; *str_p != '\0'; str_p++) {
-  }
+  str_p += len;
   
   /* tack on a '\n' if necessary */
   if (*(str_p - 1) != '\n') {
@@ -217,19 +255,20 @@ void	_dmalloc_message(const char *format, ...)
   len = str_p - str;
   
   /* do we need to log the message? */
-  if (dmalloc_logpath != LOGPATH_INIT) {
+  if (_dmalloc_logpath != LOGPATH_INIT) {
     /*
      * do we need to open the outfile?
      * it will be closed by _exit().  yeach.
      */
     if (outfile < 0) {
-      outfile = open(dmalloc_logpath, O_WRONLY | O_CREAT | O_TRUNC, 0666);
+      outfile = open(_dmalloc_logpath, O_WRONLY | O_CREAT | O_TRUNC, 0666);
       if (outfile < 0) {
-	(void)sprintf(str, "debug-malloc library: could not open '%s'\n",
-		      dmalloc_logpath);
+	(void)loc_snprintf(str, sizeof(str),
+			   "debug-malloc library: could not open '%s'\n",
+			   _dmalloc_logpath);
 	(void)write(STDERR, str, strlen(str));
 	/* disable log_path */
-	dmalloc_logpath = LOGPATH_INIT;
+	_dmalloc_logpath = LOGPATH_INIT;
 	return;
       }
       
@@ -237,22 +276,40 @@ void	_dmalloc_message(const char *format, ...)
        * NOTE: this makes it go recursive here but it will never enter
        * this section of code.
        */
-#if DMALLOC_LICENSE
-      _dmalloc_message("Dmalloc version '%s'.  Licensed copy #%d.",
-		       dmalloc_version, DMALLOC_LICENSE);
-#else
-      _dmalloc_message("Dmalloc version '%s'.  UN-LICENSED copy.",
-		       dmalloc_version);
+      _dmalloc_message("Dmalloc version '%s' from '%s'",
+		       dmalloc_version, DMALLOC_HOME);
+      _dmalloc_message("flags = %#x, logfile '%s'",
+		       _dmalloc_flags, _dmalloc_logpath);
+      if (_dmalloc_address_seen_n == ADDRESS_COUNT_INIT) {
+	_dmalloc_message("interval = %lu, addr = %#lx",
+			 _dmalloc_check_interval,
+			 (unsigned long)_dmalloc_address);
+      }
+      else {
+	_dmalloc_message("interval = %lu, addr = %#lx, seen # = %ld",
+			 _dmalloc_check_interval,
+			 (unsigned long)_dmalloc_address,
+			 _dmalloc_address_seen_n);
+      }
+#if LOCK_THREADS
+      _dmalloc_message("threads enabled, lock-init = %d", THREAD_INIT_LOCK);
 #endif
-      _dmalloc_message("dmalloc_logfile '%s': flags = %#lx, addr = %#lx",
-		       dmalloc_logpath, _dmalloc_flags, dmalloc_address);
       
 #if STORE_TIMEVAL
-      _dmalloc_message("starting time = %ld.%ld",
-		       _dmalloc_start.tv_sec, _dmalloc_start.tv_usec);
+      {
+	char	time_buf[64];
+	_dmalloc_message("starting time = %s",
+			 _dmalloc_ptimeval(&_dmalloc_start, time_buf,
+					   sizeof(time_buf), 0));
+      }
 #else
 #if HAVE_TIME /* NOT STORE_TIME */
-      _dmalloc_message("starting time = %ld", _dmalloc_start);
+      {
+	char	time_buf[64];
+	_dmalloc_message("starting time = %s",
+			 _dmalloc_ptime(&_dmalloc_start, time_buf,
+					sizeof(time_buf), 0));
+      }
 #endif
 #endif
     }
@@ -262,11 +319,24 @@ void	_dmalloc_message(const char *format, ...)
   }
   
   /* do we need to print the message? */
-  if (BIT_IS_SET(_dmalloc_flags, DEBUG_PRINT_ERROR)) {
+  if (BIT_IS_SET(_dmalloc_flags, DEBUG_PRINT_MESSAGES)) {
     (void)write(STDERR, str, len);
   }
 }
 
+/*
+ * message writer with printf like arguments
+ */
+void	_dmalloc_message(const char *format, ...)
+  /* __attribute__ ((format (printf, 1, 2))) */
+{
+  va_list	args;
+  
+  va_start(args, format);
+  _dmalloc_vmessage(format, args);
+  va_end(args);
+}
+
 /*
  * kill the program because of an internal malloc error
  */
@@ -283,12 +353,13 @@ void	_dmalloc_die(const int silent_b)
     }
     
     /* print a message that we are going down */
-    (void)sprintf(str, "debug-malloc library: %s program, fatal error\n",
-		  stop_str);
+    (void)loc_snprintf(str, sizeof(str),
+		       "debug-malloc library: %s program, fatal error\n",
+		       stop_str);
     (void)write(STDERR, str, strlen(str));
     if (dmalloc_errno != ERROR_NONE) {
-      (void)sprintf(str, "   Error: %s (err %d)\n",
-		    _dmalloc_strerror(dmalloc_errno), dmalloc_errno);
+      (void)loc_snprintf(str, sizeof(str), "   Error: %s (err %d)\n",
+			 _dmalloc_strerror(dmalloc_errno), dmalloc_errno);
       (void)write(STDERR, str, strlen(str));
     }
   }
@@ -324,8 +395,8 @@ void	_dmalloc_die(const int silent_b)
 void	dmalloc_error(const char *func)
 {
   /* do we need to log or print the error? */
-  if (dmalloc_logpath != NULL
-      || BIT_IS_SET(_dmalloc_flags, DEBUG_PRINT_ERROR)) {
+  if (_dmalloc_logpath != NULL
+      || BIT_IS_SET(_dmalloc_flags, DEBUG_PRINT_MESSAGES)) {
     
     /* default str value */
     if (func == NULL) {
