@@ -1,27 +1,24 @@
 /*
  * array of error messages for the malloc internal errors.
  *
- * Copyright 1995 by Gray Watson
+ * Copyright 1999 by Gray Watson
  *
  * This file is part of the dmalloc package.
  *
- * Permission to use, copy, modify, and distribute this software for any
- * NON-COMMERCIAL purpose and without fee is hereby granted, provided
- * that the above copyright notice and this permission notice appear
- * in all copies, and that the name of Gray Watson not be used in
- * advertising or publicity pertaining to distribution of the document
- * or software without specific, written prior permission.
- *
- * Please see the PERMISSIONS file or contact the author for information
- * about commercial licenses.
+ * Permission to use, copy, modify, and distribute this software for
+ * any purpose and without fee is hereby granted, provided that the
+ * above copyright notice and this permission notice appear in all
+ * copies, and that the name of Gray Watson not be used in advertising
+ * or publicity pertaining to distribution of the document or software
+ * without specific, written prior permission.
  *
  * Gray Watson makes no representations about the suitability of the
  * software described herein for any purpose.  It is provided "as is"
  * without express or implied warranty.
  *
- * The author may be contacted at gray.watson@letters.com
+ * The author may be contacted via http://www.dmalloc.com/
  *
- * $Id: error_str.h,v 1.21 1997/12/22 00:23:44 gray Exp $
+ * $Id: error_str.h,v 1.26 1999/03/07 23:01:53 gray Exp $
  */
 
 #ifndef __ERROR_STR_H__
@@ -44,6 +41,7 @@ static	error_str_t	error_list[] = {
   { ERROR_BAD_SETUP,		"initialization and setup failed" },
   { ERROR_IN_TWICE,		"malloc library has gone recursive" },
   { ERROR_BAD_ERRNO,		"errno value from user is out-of-bounds" },
+  { ERROR_LOCK_NOT_CONFIG,	"thread locking has not been configured" },
   
   /* pointer verification errors */
   { ERROR_IS_NULL,		"pointer is null" },
@@ -94,7 +92,10 @@ static	error_str_t	error_list[] = {
   
   /* heap check verification */
   { ERROR_BAD_BLOCK_ORDER,	"bad basic-block allocation order" },
-  { ERROR_BAD_FLAG,		"basic-block has bad flag value" }
+  { ERROR_BAD_FLAG,		"basic-block has bad flag value" },
+  
+  /* memory table errors */
+  { ERROR_TABLE_CORRUPT,	"internal memory table corruption" }
 };
 
 #endif /* ! __ERROR_STR_H__ */
