@@ -1,27 +1,24 @@
 /*
  * global error codes for chunk allocation problems
  *
- * Copyright 1995 by Gray Watson
+ * Copyright 1999 by Gray Watson
  *
  * This file is part of the dmalloc package.
  *
- * Permission to use, copy, modify, and distribute this software for any
- * NON-COMMERCIAL purpose and without fee is hereby granted, provided
- * that the above copyright notice and this permission notice appear
- * in all copies, and that the name of Gray Watson not be used in
- * advertising or publicity pertaining to distribution of the document
- * or software without specific, written prior permission.
- *
- * Please see the PERMISSIONS file or contact the author for information
- * about commercial licenses.
+ * Permission to use, copy, modify, and distribute this software for
+ * any purpose and without fee is hereby granted, provided that the
+ * above copyright notice and this permission notice appear in all
+ * copies, and that the name of Gray Watson not be used in advertising
+ * or publicity pertaining to distribution of the document or software
+ * without specific, written prior permission.
  *
  * Gray Watson makes no representations about the suitability of the
  * software described herein for any purpose.  It is provided "as is"
  * without express or implied warranty.
  *
- * The author may be contacted at gray.watson@letters.com
+ * The author may be contacted via http://www.dmalloc.com/
  *
- * $Id: error_val.h,v 1.17 1997/12/22 00:23:43 gray Exp $
+ * $Id: error_val.h,v 1.22 1999/03/07 23:01:53 gray Exp $
  */
 
 #ifndef __ERROR_VAL_H__
@@ -36,6 +33,7 @@
 #define ERROR_BAD_SETUP			10	/* bad setup value */
 #define ERROR_IN_TWICE			11	/* in malloc domain twice */
 #define ERROR_BAD_ERRNO			12	/* bad errno value */
+#define ERROR_LOCK_NOT_CONFIG		13	/* thread locking not config */
 
 /* pointer verification errors */
 #define ERROR_IS_NULL			20	/* pointer is NULL */
@@ -88,7 +86,13 @@
 #define ERROR_BAD_BLOCK_ORDER		100	/* block allocation bad */
 #define ERROR_BAD_FLAG			101	/* bad basic-block flag */
 
+/* memory table errors */
+#define ERROR_TABLE_CORRUPT		102	/* memory table corruption */
+
+/*
+ * NOTE: you must update the IS_MALLOC_ERRNO below if you add errors
+ */
 #define IS_MALLOC_ERRNO(e)	((e) >= ERROR_NONE && \
-				 (e) <= ERROR_BAD_FLAG)
+				 (e) <= ERROR_TABLE_CORRUPT)
 
 #endif /* ! __ERROR_VAL_H__ */
