@@ -1,27 +1,24 @@
 /*
  * defines for the system specific memory routines
  *
- * Copyright 1995 by Gray Watson
+ * Copyright 1999 by Gray Watson
  *
  * This file is part of the dmalloc package.
  *
- * Permission to use, copy, modify, and distribute this software for any
- * NON-COMMERCIAL purpose and without fee is hereby granted, provided
- * that the above copyright notice and this permission notice appear
- * in all copies, and that the name of Gray Watson not be used in
- * advertising or publicity pertaining to distribution of the document
- * or software without specific, written prior permission.
- *
- * Please see the PERMISSIONS file or contact the author for information
- * about commercial licenses.
+ * Permission to use, copy, modify, and distribute this software for
+ * any purpose and without fee is hereby granted, provided that the
+ * above copyright notice and this permission notice appear in all
+ * copies, and that the name of Gray Watson not be used in advertising
+ * or publicity pertaining to distribution of the document or software
+ * without specific, written prior permission.
  *
  * Gray Watson makes no representations about the suitability of the
  * software described herein for any purpose.  It is provided "as is"
  * without express or implied warranty.
  *
- * The author may be contacted at gray.watson@letters.com
+ * The author may be contacted via http://www.dmalloc.com/
  *
- * $Id: heap.h,v 1.31 1997/12/05 21:09:50 gray Exp $
+ * $Id: heap.h,v 1.35 1999/03/04 19:05:54 gray Exp $
  */
 
 #ifndef __HEAP_H__
@@ -57,6 +54,10 @@
 /* calculate the size of heap */
 #define HEAP_SIZE	((char *)_heap_last - (char *)_heap_base)
 
+/* how many blocks between the BEFORE pointer and the NOW pointer */
+#define BLOCKS_BETWEEN(now, before)	(((char *)now - (char *)before) / \
+					 BLOCK_SIZE)
+
 /* is the heap growing?  is the NEW pointer ahead of the OLD */
 #define IS_GROWTH(new, old)	((char *)(new) > (char *)(old))
 
@@ -83,6 +84,10 @@
 /* is the heap growing?  is the NEW pointer ahead of the OLD */
 #define HEAP_SIZE	((char *)_heap_base - (char *)_heap_last)
 
+/* how many blocks between the BEFORE pointer and the NOW pointer */
+#define BLOCKS_BETWEEN(now, before)	(((char *)before - (char *)now) / \
+					 BLOCK_SIZE)
+
 /* is the heap growing? */
 #define IS_GROWTH(new, old)	((char *)(new) < (char *)(old))
 
